/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * DV-Anwendungen in der Technik - Prof.Dr.Gleich - 8.Semester
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 5.0
 * Semestergroup: 8T
 */

/**
 * USABILITY:
 *  Copyright (c) 2005 
 *  Thomas Woellert (thomas@woellert.net)
 *  Stefan Weissbach (weissbst@gmx.de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package fhm.edu.fem.animation;

import java.awt.Color;

/**
 * Class Description:
 *		This class extends itself from the Java Color class and overrides the getRed, getGreen, getBlue
 *		Methods to actually serve color values from (0.0 - 1.0) instead of (0 - 255)
 * 
 * @author Thomas Woellert (thomas.woellert@t-online.de)
 * @author Stefan Weissbach (weissbst@gmx.de)
 */
public class GLColor extends Color {

	/**
	 * Constructor
	 * @param r, the red component of the color (0 - 255)
	 * @param g, the green component of the color (0 - 255)
	 * @param b, the blue component of the color (0 - 255)
	 */
	public GLColor(int r, int g, int b) {
		super(r, g, b);
	}

	/**
	 * Get the blue color as 0.0 - 1.0 value
	 * @return the color
	 */
	public float getBlueGL() {
		
		return transformColorValue(super.getBlue());
	}

	/**
	 * Get the green color as 0.0 - 1.0 value
	 * @return the color
	 */
	public float getGreenGL() {
		
		return transformColorValue(super.getGreen());
	}

	/**
	 * Get the red color as 0.0 - 1.0 value
	 * @return the color
	 */
	public float getRedGL() {
		
		return transformColorValue(super.getRed());
	}
	
	/**
	 * Transform the color value from 0 - 255 to 0.0 - 1.0
	 * @param value, the 0 - 255 color value
	 * @return the 0.0 - 1.0 color value
	 */
	protected float transformColorValue(double value) {
		
		// Determine the steps
		double steps = 1.0 / 255;
		return (float)(steps*value);
	}
}
