/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * DV-Anwendungen in der Technik - Prof.Dr.Gleich - 8.Semester
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 5.0
 * Semestergroup: 8T
 */

/**
 * USABILITY:
 *  Copyright (c) 2005 
 *  Thomas Woellert (thomas@woellert.net)
 *  Stefan Weissbach (weissbst@gmx.de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package fhm.edu.fem.animation;

import java.awt.Frame;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import net.java.games.jogl.Animator;
import net.java.games.jogl.GLCanvas;
import net.java.games.jogl.GLCapabilities;
import net.java.games.jogl.GLDrawableFactory;
import fhm.edu.fem.calc.objects.Vector;
import fhm.edu.fem.constants.ApplicationConstants;

/**
 * Class Description:
 *		This class acts as the main frame for the GLMembraneRenderer
 * 
 * @author Thomas Woellert (thomas.woellert@t-online.de)
 * @author Stefan Weissbach (weissbst@gmx.de)
 */
public class GLMembraneFrame extends Frame {
	
	// The renderer used by this frame to display the membrane
	protected GLMembraneRenderer membraneRenderer = null;

	/**
	 * Constructor
	 * @param title, the title displayed on top of the frame
	 */
	public GLMembraneFrame(String title, double dimensionA, double dimensionB, Vector w0) {
		super(title);

		// Create the renderer with the given options
		membraneRenderer = new GLMembraneRenderer(dimensionA, dimensionB, w0);
		
		// Initialize the frame
		this.init();
	}
	
	/**
	 * Initialize the frame
	 */
	protected void init() {
		
		// Set the frame's size
        this.setSize(ApplicationConstants.ANIMATION_FRAME_WIDTH, ApplicationConstants.ANIMATION_FRAME_HEIGHT);
        
        // Create the main canvas object
	    GLCanvas canvas = GLDrawableFactory.getFactory().createGLCanvas(new GLCapabilities());
	    
	    // Tell our renderer as event listener to the canvas and the frame
	    canvas.addGLEventListener(this.membraneRenderer);
	    
	    // Add the canvas to this frame so it can be displayed
	    this.add(canvas);
	    
	    // Add the renderer as keylistener to the canvas (both due to a bug)
	    canvas.addKeyListener(this.membraneRenderer);
	    this.addKeyListener(this.membraneRenderer);

	    // Create a new animator to display the changes in the membrane
	    final Animator animator = new Animator(canvas);
	    
	    // Make sure that the Animator is properly closed and it's Thread stopped before exiting the program
	    this.addWindowListener(new WindowAdapter() {
	    	
	    	// Exit the thread first before closing this window
	        public void windowClosing(WindowEvent e) {
	          
	        	// Create a new Thread to make sure the animator is properly stopped and start it
	        	new Thread(new Runnable() {
	        		
	              public void run() {
	                animator.stop();
	                System.exit(0);
	              }
	              
	            }).start();
	        }
	      });
	    
	    // Show the frame
	    this.setVisible(true);
	    
	    // Start the animation
	    animator.start();
	}
	
	/**
	 * Set a new vector w(t) while running
	 * @param wT, the new Vector wT
	 */
	public void setNewVectorW(Vector wT) {
		membraneRenderer.setNewVectorW(wT);
	}
}
