/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * DV-Anwendungen in der Technik - Prof.Dr.Gleich - 8.Semester
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 5.0
 * Semestergroup: 8T
 */

/**
 * USABILITY:
 *  Copyright (c) 2005 
 *  Thomas Woellert (thomas@woellert.net)
 *  Stefan Weissbach (weissbst@gmx.de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package fhm.edu.fem.app;

import java.lang.reflect.Method;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import fhm.edu.fem.calc.FEMGaussQuadrature;
import fhm.edu.fem.calc.objects.Functions;
import fhm.edu.fem.calc.objects.MathObject;
import fhm.edu.fem.calc.objects.Vector;
import fhm.edu.fem.calc.objects.matrix.Matrix;
import fhm.edu.fem.constants.ApplicationConstants;
import fhm.edu.fem.constants.GaussConstants;
import fhm.edu.fem.constants.PropertyConstants;
import fhm.edu.fem.util.logging.LoggingManager;
import fhm.edu.fem.util.properties.PropertyCache;

/**
 * Class Description:
 *		To start the GaussQuadrature of the 2nd "Leistungsnachweis"
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 * @author Stefan Weissbach (weissbst@gmx.de)
 */
public class GaussQuadratureStartup {
    
	/* Our local logger object: Null at the beginning, because we need to initialize the LoggingManager first */
	protected static Logger localLogger = null;
	
	/* Inner and Outer Points ("Stuetzstellen des inneren und aeusseren Integrals") */
	protected static int INNER_POINTS = 8;
	protected static int OUTER_POINTS = 6;
	
	/**
	 * Call this method to start the GaussQuadrature from the 2nd "Leistungsnachweis"
	 * @param args, command line arguments, none needed
	 */
    public static void main(String[] args) {
		try {
			// Initialize the logging manager
			LoggingManager.setProperties(PropertyCache.getProperties(PropertyConstants.GLOBAL_PROPS));
			
			// Now we can initialize our local logger
			localLogger = LoggingManager.getLogger(GaussQuadratureStartup.class.getName());
			
			// Check if we properly received a logger
			if(localLogger != null) {
				
				if(localLogger.isEnabledFor(Level.INFO)) {					
					// Print startup text
					System.out.println(ApplicationConstants.getStartupText());
					
					localLogger.log(Level.INFO, "Starting GaussQuadrature ...");
				}
				
				// Separate the command-line arguments and start the application
				GaussQuadratureStartup.start();
				
			} else {
				System.err.println("[FATAL] - LoggingManager and/or Logger initialization failed - Aborting !");
			}
		} catch (Exception e) {
			if(localLogger != null) {
				if(localLogger.isEnabledFor(Level.ERROR)) {
					localLogger.log(Level.ERROR, "Running Startup failed!", e);
				}
			} else {
				System.err.println("[FATAL] - Running Startup failed !");
				e.printStackTrace();
			}
		}
    }
    
    /**
     * Start the GaussQuadrature
     */
    public static void start() {
        
        try {
          
            // The variables to use
            double a = -1;
            double b = -1;
            Method h1 = null;
            Method h2 = null;
            Method q = null;
            int z1 = -1;
            int z2 = -1;
            Vector ax = null;
            Vector hx = null;
            Vector ay = null;
            Vector hy = null;
            int figure = -1;
            MathObject resultObject = null;
            
    		if(localLogger.isEnabledFor(Level.INFO)) {
    			localLogger.log(Level.INFO, "Calculating (a) ...");
    		}
    		
    		a = 0;
    		b = 1;
    		h1 = Functions.getMethod(Functions.PART_A_H1);
    		h2 = Functions.getMethod(Functions.PART_A_H2);
    		q = Functions.getMethod(Functions.PART_A_Q);
    		z1 = 1;
    		z2 = 1;
    		ax = GaussConstants.get(INNER_POINTS, GaussConstants.VALUE_A);
    		hx = GaussConstants.get(INNER_POINTS, GaussConstants.VALUE_H);
    		ay = GaussConstants.get(OUTER_POINTS, GaussConstants.VALUE_A);
    		hy = GaussConstants.get(OUTER_POINTS, GaussConstants.VALUE_H);
    		figure = GaussConstants.FIGURE_TRIANGLE;
    		
    		resultObject = FEMGaussQuadrature.calcIntegration(a, b, h1, h2, q, z1, z2, ax, hx, ay, hy, figure);
            
    		if(localLogger.isEnabledFor(Level.INFO)) {
    			localLogger.log(Level.INFO, "Result of (a): " + resultObject);
    		}
    		
    		if(localLogger.isEnabledFor(Level.INFO)) {
    			localLogger.log(Level.INFO, "--------------------------------");
    		}
    		
    		if(localLogger.isEnabledFor(Level.INFO)) {
    			localLogger.log(Level.INFO, "Calculating (b) ...");
    		}
    		
    		a = 0;
    		b = 1;
    		h1 = Functions.getMethod(Functions.PART_B_H1);
    		h2 = Functions.getMethod(Functions.PART_B_H2);
    		q = Functions.getMethod(Functions.PART_B_Q);
    		z1 = 3;
    		z2 = 3;
    		ax = GaussConstants.get(INNER_POINTS, GaussConstants.VALUE_A);
    		hx = GaussConstants.get(INNER_POINTS, GaussConstants.VALUE_H);
    		ay = GaussConstants.get(OUTER_POINTS, GaussConstants.VALUE_A);
    		hy = GaussConstants.get(OUTER_POINTS, GaussConstants.VALUE_H);
    		figure = GaussConstants.FIGURE_TRIANGLE;
    		
    		resultObject = FEMGaussQuadrature.calcIntegration(a, b, h1, h2, q, z1, z2, ax, hx, ay, hy, figure);
            
    		if(localLogger.isEnabledFor(Level.INFO)) {
    			localLogger.log(Level.INFO, "Result of (b): " + resultObject);
    		}
    		
    		if(localLogger.isEnabledFor(Level.INFO)) {
    			localLogger.log(Level.INFO, "--------------------------------");
    		}
    		
    		if(localLogger.isEnabledFor(Level.INFO)) {
    			localLogger.log(Level.INFO, "Calculating (c) ...");
    		}
    		
    		a = 0;
    		b = 1;
    		h1 = Functions.getMethod(Functions.PART_C_H1);
    		h2 = Functions.getMethod(Functions.PART_C_H2);
    		q = Functions.getMethod(Functions.PART_C_Q);
    		z1 = 2;
    		z2 = 2;
    		ax = GaussConstants.get(INNER_POINTS, GaussConstants.VALUE_A);
    		hx = GaussConstants.get(INNER_POINTS, GaussConstants.VALUE_H);
    		ay = GaussConstants.get(OUTER_POINTS, GaussConstants.VALUE_A);
    		hy = GaussConstants.get(OUTER_POINTS, GaussConstants.VALUE_H);
    		figure = GaussConstants.FIGURE_TRIANGLE;
    		
    		resultObject = FEMGaussQuadrature.calcIntegration(a, b, h1, h2, q, z1, z2, ax, hx, ay, hy, figure);
            
    		if(localLogger.isEnabledFor(Level.INFO)) {
    			localLogger.log(Level.INFO, "Result of (c): " + resultObject);
    		}
    		
    		if(localLogger.isEnabledFor(Level.INFO)) {
    			localLogger.log(Level.INFO, "--------------------------------");
    		}
    		
    		if(localLogger.isEnabledFor(Level.INFO)) {
    			localLogger.log(Level.INFO, "Calculating (d) ...");
    		}
    		
    		a = 0;
    		b = 1;
    		h1 = Functions.getMethod(Functions.PART_D_H1);
    		h2 = Functions.getMethod(Functions.PART_D_H2);
    		q = Functions.getMethod(Functions.PART_D_Q);
    		z1 = 3;
    		z2 = 1;
    		ax = GaussConstants.get(INNER_POINTS, GaussConstants.VALUE_A);
    		hx = GaussConstants.get(INNER_POINTS, GaussConstants.VALUE_H);
    		ay = GaussConstants.get(OUTER_POINTS, GaussConstants.VALUE_A);
    		hy = GaussConstants.get(OUTER_POINTS, GaussConstants.VALUE_H);
    		figure = GaussConstants.FIGURE_TRIANGLE;
    		
    		resultObject = FEMGaussQuadrature.calcIntegration(a, b, h1, h2, q, z1, z2, ax, hx, ay, hy, figure);
            
    		if(localLogger.isEnabledFor(Level.INFO)) {
    			localLogger.log(Level.INFO, "Result of (d): " + resultObject);
    		}
    		
    		if(localLogger.isEnabledFor(Level.INFO)) {
    			localLogger.log(Level.INFO, "--------------------------------");
    		}
    		
    		if(localLogger.isEnabledFor(Level.INFO)) {
    			localLogger.log(Level.INFO, "Calculating (e) ...");
    		}
    		
    		// Calculate the first integral of (e)
    		
    		a = 0;
    		b = 1;
    		h1 = Functions.getMethod(Functions.PART_E_H1);
    		h2 = Functions.getMethod(Functions.PART_E_H2);
    		q = Functions.getMethod(Functions.PART_E_Q);
    		z1 = 6;
    		z2 = 6;
    		ax = GaussConstants.get(INNER_POINTS, GaussConstants.VALUE_A);
    		hx = GaussConstants.get(INNER_POINTS, GaussConstants.VALUE_H);
    		ay = GaussConstants.get(OUTER_POINTS, GaussConstants.VALUE_A);
    		hy = GaussConstants.get(OUTER_POINTS, GaussConstants.VALUE_H);
    		figure = GaussConstants.FIGURE_TRIANGLE;
    		
    		resultObject = FEMGaussQuadrature.calcIntegration(a, b, h1, h2, q, z1, z2, ax, hx, ay, hy, figure);
    		
    		// Calculate the second integral of (e)
    		h1 = Functions.getMethod(Functions.PART_E_H2);
    		h2 = Functions.getMethod(Functions.PART_E_H1);
    		
    		Object resultObject2 = FEMGaussQuadrature.calcIntegration(a, b, h1, h2, q, z1, z2, ax, hx, ay, hy, figure);
    		
    		// Add both results together (at this point we know both are matrices, so simply cast to that type)
    		resultObject = ((Matrix)resultObject).add((Matrix)resultObject2);
            
    		if(localLogger.isEnabledFor(Level.INFO)) {
    			localLogger.log(Level.INFO, "Result of (e): " + resultObject);
    		}
    		
    		if(localLogger.isEnabledFor(Level.INFO)) {
    			localLogger.log(Level.INFO, "--------------------------------");
    		}
    		
    		if(localLogger.isEnabledFor(Level.INFO)) {
    			localLogger.log(Level.INFO, "Calculating (f) ...");
    		}
    		
    		a = 0;
    		b = 1;
    		h1 = Functions.getMethod(Functions.PART_F_H1);
    		h2 = Functions.getMethod(Functions.PART_F_H2);
    		q = Functions.getMethod(Functions.PART_F_Q);
    		z1 = 3;
    		z2 = 3;
    		ax = GaussConstants.get(INNER_POINTS, GaussConstants.VALUE_A);
    		hx = GaussConstants.get(INNER_POINTS, GaussConstants.VALUE_H);
    		ay = GaussConstants.get(OUTER_POINTS, GaussConstants.VALUE_A);
    		hy = GaussConstants.get(OUTER_POINTS, GaussConstants.VALUE_H);
    		figure = GaussConstants.FIGURE_QUADRAT;
    		
    		resultObject = FEMGaussQuadrature.calcIntegration(a, b, h1, h2, q, z1, z2, ax, hx, ay, hy, figure);
            
    		if(localLogger.isEnabledFor(Level.INFO)) {
    			localLogger.log(Level.INFO, "Result of (f): " + resultObject);
    		}
    		
    		if(localLogger.isEnabledFor(Level.INFO)) {
    			localLogger.log(Level.INFO, "--------------------------------");
    		}
    		
        } catch (Exception e) {
            
    		if(localLogger.isEnabledFor(Level.ERROR)) {
    			localLogger.log(Level.ERROR, "Error during calculation !", e);
    		}
        }
    }
    
	/**
	 * Method called when aborting the program
	 */
	public static void abort() {
	    
	    // Exit the application
	    System.exit(0);
	}
}
