/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * DV-Anwendungen in der Technik - Prof.Dr.Gleich - 8.Semester
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 5.0
 * Semestergroup: 8T
 */

/**
 * USABILITY:
 *  Copyright (c) 2005 
 *  Thomas Woellert (thomas@woellert.net)
 *  Stefan Weissbach (weissbst@gmx.de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package fhm.edu.fem.calc.objects;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * Class Description:
 *		This class acts as border list. It takes the border index from the border itself
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 * @author Stefan Weissbach (weissbst@gmx.de)
 */
public class BorderList {

	/* The List object where we store our borders */
	protected ArrayList<Border> borders = new ArrayList<Border>();
	
	/**
	 * Create a new border list
	 */
	public BorderList() {}
	
	/**
	 * Set a new border in the list. It will be set at the index specified in the border
	 * @param border, the border to set
	 */
	public void setBorder(Border border) {
		borders.add(border);
	}
	
	/**
	 * Retrieve a border with the given index from the list
	 * @param borderIndex
	 * @return the found border
	 */
	public Border getBorder(int borderIndex) {
		
		// Run through the list and find the border with the given index
		Iterator it = borders.iterator();
		Border tmpBorder = null;
		
		while(it.hasNext()) {
			// Grab the next border from the list
			tmpBorder = (Border)(it.next());
			
			// Check if it is the border we are searching for
			if(tmpBorder != null) {
				
				if(borderIndex == tmpBorder.getId()) {
					
					// We found the correct border, return it
					return tmpBorder;
				}
			}
		}
		
		// We found no proper border
		return null;
	}
	
	/**
	 * Convert this list to an array of borders
	 * @return the border array
	 */
	public Object[] toArray() {
		return this.borders.toArray();
	}
	
	/**
	 * Clear the list
	 */
	public void clear() {
		this.borders.clear();
	}
	
	/**
	 * Retrieve the number of borders in the list
	 * @return the number of borders
	 */
	public int size() {
		return this.borders.size();
	}
	
	/**
	 * Clone Implementation
	 * @return a clone of this list
	 */
	public Object clone() {
		// Create the new list
		BorderList cloneList = new BorderList();
		
		// Fill the new list with the same borders as the current list
		Object[] borders = this.toArray();
		
		for(int i = 0; i < borders.length; i++) {
			cloneList.setBorder((Border)(borders[i]));
		}
		
		return cloneList;
	}
}
