/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * DV-Anwendungen in der Technik - Prof.Dr.Gleich - 8.Semester
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 5.0
 * Semestergroup: 8T
 */

/**
 * USABILITY:
 *  Copyright (c) 2005 
 *  Thomas Woellert (thomas@woellert.net)
 *  Stefan Weissbach (weissbst@gmx.de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package fhm.edu.fem.calc.objects;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import fhm.edu.fem.util.logging.LoggingManager;

/**
 * Class Description:
 *		This class represents a "ctab"
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 * @author Stefan Weissbach (weissbst@gmx.de)
 */
public class CTab {
    
	/* Our local logger object */
	protected static Logger localLogger = LoggingManager.getLogger(CTab.class.getName());

	/* The nodes forming the CTab */
    protected Node midNode = null;
    protected Node leftNode = null;
    protected Node rightNode = null;
    
    /* The ID of the CTab */
    protected int id = -1;
    
    /**
     * Construct a new CTab
     * @param id, the ID number
     * @param leftNode, the left node
     * @param midNode, the middle node
     * @param rightNode, the right node
     */
    public CTab(int id, Node leftNode, Node midNode, Node rightNode) {
        this.id = id;
        this.midNode = midNode;
        this.leftNode = leftNode;
        this.rightNode = rightNode;
    }
    
    /**
     * @return Returns the id.
     */
    public int getId() {
    	return this.id;
    }
    
    /**
     * @return Returns the left node.
     */
    public Node getLeftNode() {
        return this.leftNode;
    }
    
    /**
     * @return Returns the right node.
     */
    public Node getRightNode() {
        return this.rightNode;
    }
    
    /**
     * @return Returns the mid node.
     */
    public Node getMidNode() {
        return this.midNode;
    }
    
    /**
     * Returns the node depending on i
     * usage: i = 1 (left node), 2 (mid node), 3 (right node)
     * @param i, may be 1 (left node), 2 (mid node), 3 (right node)
     * @return the requested node
     */
    public Node getNode(int i) {
        
        switch(i) {
        
        	case 1:
        	    return this.leftNode;
        	    
        	case 2:
        	    return this.midNode;
        	        
        	case 3:
        	    return this.rightNode;
        	
        	default:
                if(localLogger.isEnabledFor(Level.ERROR)) {
                    localLogger.log(Level.ERROR, "Error: Unknown number to get a node, usage: 1 = left, 2 = mid, 3 = right !");
                }
        }
        
        return null;
    }
    
    /**
     * Implementation of the toString() method
     * @return a string visualization of the element
     */
    public String toString() {
        return  "[CTab id=" + this.id + " left_node_id = " + leftNode.getId() + " mid_node_id = " + midNode.getId() + " right_node_id = " + rightNode.getId() + "]";
    }
}
