/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * DV-Anwendungen in der Technik - Prof.Dr.Gleich - 8.Semester
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 5.0
 * Semestergroup: 8T
 */

/**
 * USABILITY:
 *  Copyright (c) 2005 
 *  Thomas Woellert (thomas@woellert.net)
 *  Stefan Weissbach (weissbst@gmx.de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package fhm.edu.fem.calc.objects;

import java.util.ArrayList;
import java.util.Iterator;

import org.apache.log4j.Logger;

import fhm.edu.fem.util.logging.LoggingManager;

/**
 * Class Description:
 *		This class acts as ctab list. It takes the ctab index from the ctab itself
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 * @author Stefan Weissbach (weissbst@gmx.de)
 */
public class CTabList {
    
	/* Our local logger object */
	protected static Logger localLogger = LoggingManager.getLogger(CTabList.class.getName());

	/* The List object where we store our elements */
	protected ArrayList<CTab> ctabs = new ArrayList<CTab>();
	
	/**
	 * Create a new ctab list
	 */
	public CTabList() {}
	
	/**
	 * Set a new ctab in the list. It will be set at the index specified in the ctab
	 * @param ctab, the ctab to set
	 */
	public void setCTab(CTab ctab) {
	    ctabs.add(ctab);
	}
	
	/**
	 * Retrieve a ctab with the given index from the list
	 * @param ctabIndex
	 * @return the found ctab
	 */
	public CTab getCTab(int ctabIndex) {
		
		// Run through the list and find the element with the given index
		Iterator it = ctabs.iterator();
		CTab tmpCTab = null;
		
		while(it.hasNext()) {
			// Grab the next ctab from the list
			tmpCTab = (CTab)(it.next());
			
			// Check if it is the ctab we are searching for
			if(tmpCTab != null) {
				
				if(ctabIndex == tmpCTab.getId()) {
					
					// We found the correct ctab, return it
					return tmpCTab;
				}
			}
		}
		
		// We found no proper ctab
		return null;
	}
	
	/**
	 * Convert this list to an array of ctab
	 * @return the ctab array
	 */
	public Object[] toArray() {
		return this.ctabs.toArray();
	}
	
	/**
	 * Clear the list
	 */
	public void clear() {
		this.ctabs.clear();
	}
	
	/**
	 * Retrieve the number of ctabs in the list
	 * @return the number of ctabs
	 */
	public int size() {
		return this.ctabs.size();
	}
	
	/**
	 * Clone Implementation
	 * @return a clone of this list
	 */
	public Object clone() {
		// Create the new list
		CTabList cloneList = new CTabList();
		
		// Fill the new list with the same ctabs as the current list
		Object[] ctabArray = this.toArray();
		
		for(int i = 0; i < ctabArray.length; i++) {
			cloneList.setCTab((CTab)(ctabArray[i]));
		}
		
		return cloneList;
	}
}
