/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * DV-Anwendungen in der Technik - Prof.Dr.Gleich - 8.Semester
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 5.0
 * Semestergroup: 8T
 */

/**
 * USABILITY:
 *  Copyright (c) 2005 
 *  Thomas Woellert (thomas@woellert.net)
 *  Stefan Weissbach (weissbst@gmx.de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package fhm.edu.fem.calc.objects;

import java.util.List;

/**
 * Class Description:
 *		This class acts as super container of a calculation holding the calculation's conditions
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 * @author Stefan Weissbach (weissbst@gmx.de)
 */
public class Calculation {

	/* The id of this calculation */
	protected int id = -1;
	
    /* The conditions belonging to this calculation */
    protected List<Condition> conditions = null;
    
    /**
     * Construct a new Calculation
     * @param id, the ID number of the calculation
     * @param conditions, the conditions this calculation needs
     */
    public Calculation(int id, List<Condition> conditions) {
    	this.id = id;
        this.conditions = conditions;
    }
    
    /**
     * Retrieve the conditions from this calculation
     * @return the condition list
     */
    public List<Condition> getConditions() {
        return this.conditions;
    }
    
    /**
     * Implementation of the toString() method
     * @return a string visualization of the element
     */
    public String toString() {
        String string =  "[Calculation id=" + this.id + " number_of_conditions=" + this.conditions.size() + " conditions=";
        
        for(int i = 0; i < this.conditions.size(); i++) {
            string = string + " " + this.conditions.get(i);
        }
        
        return string + "]";
    }
}
