/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * DV-Anwendungen in der Technik - Prof.Dr.Gleich - 8.Semester
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 5.0
 * Semestergroup: 8T
 */

/**
 * USABILITY:
 *  Copyright (c) 2005 
 *  Thomas Woellert (thomas@woellert.net)
 *  Stefan Weissbach (weissbst@gmx.de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package fhm.edu.fem.calc.objects;

import org.apache.log4j.Logger;

import fhm.edu.fem.constants.SMatrixConstants;
import fhm.edu.fem.util.logging.LoggingManager;

/**
 * Class Description:
 *		This class acts as "element"
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 * @author Stefan Weissbach (weissbst@gmx.de)
 */
public class Element {

	/* Our local logger object */
	protected static Logger localLogger = LoggingManager.getLogger(Element.class.getName());
    
    /* The id identifying this element */
    protected int id = -1;
    
    /* The list of nodes this element consists of */
    protected NodeList nodeList = null;

    /* The type of the element, either TRI or QUAD */
    protected int elementType = -1;
    
    /* The calculated "Spannungsbetrag" for this element */
    protected double spannungsBetrag = -1;
    
    /**
     * Construct a new element with the given ID, consisting of the given nodes
     * @param id, the ID for this node
     * @param elementType, may be FIGURE_TRIANGLE or FIGURE_PARALLELOGRAM
     * @param nodeList, the list of nodes belonging to this element in counter-clockwise order
     */
    public Element(int id, int elementType, NodeList nodeList) {
        this.id = id;
        this.nodeList = nodeList;
        this.elementType = elementType;
        
        // Set the element type according to the nodelist size
        if(nodeList.size() == 3 || nodeList.size() == 6) {
        	
        	this.elementType = SMatrixConstants.FIGURE_TRIANGLE;
        	
        } else if(nodeList.size() == 4 || nodeList.size() == 8) {
        	
        	this.elementType = SMatrixConstants.FIGURE_QUADRAT;
        }
    }
    
    /**
     * @return Returns the id.
     */
    public int getId() {
        return id;
    }
    
    /**
     * @return Returns the nodeList.
     */
    public NodeList getNodeList() {
        return this.nodeList;
    }
    
    /**
     * Retrieve the element type
     * @return either FIGURE_TRIANGLE or FIGURE_PARALLELOGRAM
     */
    public int getElementType() {
        return this.elementType;
    }
    
    /**
     * Implementation of the toString() method
     * @return a string visualization of the element
     */
    public String toString() {
        String string =  "[Element id=" + this.id + " number_of_nodes=" + this.nodeList.size() + " node_ids=";
        
        for(int i = 1; i <= this.nodeList.size(); i++) {
            string = string + " " + this.nodeList.getNodeInCorrectOrder(i).getId();
        }
        
        return string + "]";
    }
    
	/**
	 * @return Returns the spannungsBetrag.
	 */
	public double getSpannungsBetrag() {
		return spannungsBetrag;
	}
	
	/**
	 * @param spannungsBetrag The spannungsBetrag to set.
	 */
	public void setSpannungsBetrag(double spannungsBetrag) {
		this.spannungsBetrag = spannungsBetrag;
	}
}
