/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * DV-Anwendungen in der Technik - Prof.Dr.Gleich - 8.Semester
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 5.0
 * Semestergroup: 8T
 */

/**
 * USABILITY:
 *  Copyright (c) 2005 
 *  Thomas Woellert (thomas@woellert.net)
 *  Stefan Weissbach (weissbst@gmx.de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package fhm.edu.fem.calc.objects;

/**
 * Class Description:
 *		This class acts as "force"
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 * @author Stefan Weissbach (weissbst@gmx.de)
 */
public class Force {

    /* The id identifying this force */
    protected int id = -1;
    
    /* The x-value of this force */
    protected double x = -1;
    
    /* The y-value of this force */
    protected double y = -1;
    
    /* The node this force is related to */
    protected Node node = null;
    
    /**
     * Construct a new force with the given ID
     * @param id, the ID for this force
     * @param x, the x coordinate for this force
     * @param y, the y coordinate for this force
     * @param node, the node this force is related to
     */
    public Force(int id, double x, double y, Node node) {
        this.id = id;
        this.x = x;
        this.y = y;
        this.node = node;
    }
    
    /**
     * @return Returns the id.
     */
    public int getId() {
        return id;
    }
    
    /**
     * @return Returns the x.
     */
    public double getX() {
        return x;
    }
    
    /**
     * @return Returns the y.
     */
    public double getY() {
        return y;
    }
    
	/**
	 * @param x The x to set.
	 */
	public void setX(double x) {
		this.x = x;
	}
	
	/**
	 * @param y The y to set.
	 */
	public void setY(double y) {
		this.y = y;
	}
	
	/**
	 * @return Returns the node.
	 */
	public Node getNode() {
		return node;
	}
	
    /**
     * Implementation of the toString() method
     * @return a string visualization of the node
     */
    public String toString() {
        
        return "[Force id=" + this.id + " x=" + this.x + " y=" + this.y + " Node=" + this.node + " ]";
    }
    
    /**
     * Compare this force with another given force
     * @param obj, the other force
     */
    public boolean equals(Object obj) {
        
        // Check if the obj is an instance of force
        if(!(obj instanceof Force)) {
            
            return false;
        }
        
        // We got a force, compare it
        Force force = (Force)(obj);
        
        // Compare everything
        if(this.id != force.getId()) {
            
            return false;
            
        } else if(this.x != force.getX()) {
            
            return false;
            
        } else if(this.y != force.getY()) {
            
            return false;
            
        } else if(this.node != force.getNode()) {
            
            return false;
        }
        
        return true;
    }
}
