/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * DV-Anwendungen in der Technik - Prof.Dr.Gleich - 8.Semester
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 5.0
 * Semestergroup: 8T
 */

/**
 * USABILITY:
 *  Copyright (c) 2005 
 *  Thomas Woellert (thomas@woellert.net)
 *  Stefan Weissbach (weissbst@gmx.de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package fhm.edu.fem.calc.objects;

import java.util.ArrayList;
import java.util.Iterator;

/**
 * Class Description:
 *		This class acts as force list. It takes the node index from the node itself
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 * @author Stefan Weissbach (weissbst@gmx.de)
 */
public class ForceList {

	/* The List object where we store our forces */
	protected ArrayList<Force> forces = new ArrayList<Force>();
	
	/**
	 * Create a new force list
	 */
	public ForceList() {}
	
	/**
	 * Search for a specific force
	 * @param force, the force to search
	 * @return boolean, true if it contains the list, false otherwise
	 */
	public boolean contains(Force force) {
		return this.forces.contains(force);
	}
	
	/**
	 * @return Get the ArrayList
	 */
	public ArrayList<Force> getList() {	
		return this.forces;
	}
	
	/**
	 * Set a new force in the list. It will be set at the index specified in the force
	 * @param force, the force to set
	 */
	public void setForce(Force force) {
		forces.add(force);
	}
	
	/**
	 * Retrieve a force with the given index from the list
	 * @param forceIndex
	 * @return the found force or 'null' if no force has been found
	 */
	public Force getForce(int forceIndex) {
		
		// Run through the list and find the force with the given index
		Iterator it = forces.iterator();
		Force tmpForce = null;
		
		while(it.hasNext()) {
			// Grab the next force from the list
			tmpForce = (Force)(it.next());
			
			// Check if it is the force we are searching for
			if(tmpForce != null) {
				
				if(forceIndex == tmpForce.getId()) {
					
					// We found the correct force, return it
					return tmpForce;
				}
			}
		}
		
		// We found no proper force
		return null;
	}
	
	/**
	 * Clear the list
	 */
	public void clear() {
		this.forces.clear();
	}
	
	/**
	 * Convert this list to an array of forces
	 * @return the node array
	 */
	public Object[] toArray() {
		return this.forces.toArray();
	}
	
	/**
	 * Retrieve the number of forces in the list
	 * @return the number of forces
	 */
	public int size() {
		return this.forces.size();
	}
}
