/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * DV-Anwendungen in der Technik - Prof.Dr.Gleich - 8.Semester
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 5.0
 * Semestergroup: 8T
 */

/**
 * USABILITY:
 *  Copyright (c) 2005 
 *  Thomas Woellert (thomas@woellert.net)
 *  Stefan Weissbach (weissbst@gmx.de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package fhm.edu.fem.calc.objects;

import java.lang.reflect.Method;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import fhm.edu.fem.exception.FunctionCalculationException;
import fhm.edu.fem.util.logging.LoggingManager;

/**
 * Class Description:
 *		Holds all the static function methods needed for the 2nd "Leistungsnachweis"
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 * @author Stefan Weissbach (weissbst@gmx.de)
 */
public  class Functions {
    
	/* Our local logger object */
	protected static Logger localLogger = LoggingManager.getLogger(Functions.class.getName());
	
	/* The method names */
	public static final String PART_A_H1 = "partAh1";
	public static final String PART_A_H2 = "partAh2";
	public static final String PART_A_Q = "partAq";
	
	public static final String PART_B_H1 = "partBh1";
	public static final String PART_B_H2 = "partBh2";
	public static final String PART_B_Q = "partBq";
	
	public static final String PART_C_H1 = "partCh1";
	public static final String PART_C_H2 = "partCh2";
	public static final String PART_C_Q = "partCq";
	
	public static final String PART_D_H1 = "partDh1";
	public static final String PART_D_H2 = "partDh2";
	public static final String PART_D_Q = "partDq";
	
	public static final String PART_E_H1 = "partEh1";
	public static final String PART_E_H2 = "partEh2";
	public static final String PART_E_Q = "partEq";
	
	public static final String PART_F_H1 = "partFh1";
	public static final String PART_F_H2 = "partFh2";
	public static final String PART_F_Q = "partFq";
	
	public static final String ZERO = "zero";
    
	/**
	 * Get the proper method
	 * @param methodName, the method name taken from the constants in this class
	 * @return the method
	 */
	public static Method getMethod(String methodName) throws SecurityException, NoSuchMethodException {
	    
	    // Build the args array
	    Class args[] = new Class[3];
	    args[0] = Double.class;
	    args[1] = Double.class;
	    args[2] = Integer.class;
	    
	    return Functions.class.getDeclaredMethod(methodName, args);
	}
	
	/**
	 * 
	 * @param xi
	 * @param eta
	 * @param position
	 * @return
	 */
	public static double zero(Double xi, Double eta, Integer position) {
	    
	    return 0.0;
	}
	
    /**
     * Function for part A H1
     * @param xi
     * @param eta
     * @return
     */
    public static double partAh1(Double xi, Double eta, Integer position) throws FunctionCalculationException {

        return xi.doubleValue() * eta.doubleValue();
    }
    
    /**
     * Function for part A H2
     * @param xi
     * @param eta
     * @return
     */
    public static double partAh2(Double xi, Double eta, Integer position) throws FunctionCalculationException {

        return Functions.partAh1(xi, eta, position);
    }
    
    /**
     * Function for part A q
     * @param xi
     * @param eta
     * @return
     */
    public static double partAq(Double xi, Double eta, Integer position) throws FunctionCalculationException {
        
        return 1.0;
    }
    
    /**
     * Function for part B H1
     * @param xi
     * @param eta
     * @return
     */
    public static double partBh1(Double xi, Double eta, Integer position) throws FunctionCalculationException {
        
        // Determine which position in the vector the user wants to calculate
        switch(position.intValue()) {
        
        	case 1:
        	    
        	    return (1.0 - xi.doubleValue() - eta.doubleValue());
        	
        	case 2:

        	    return (xi.doubleValue());
        	
        	case 3:
        	    
        	    return (eta.doubleValue());
        	
        	default:
                if(localLogger.isEnabledFor(Level.ERROR)) {
                    localLogger.log(Level.ERROR, "Error: Unknown position in function vector given !");
                }
                
                throw new FunctionCalculationException("Illegal position value !");
        }
    }
    
    /**
     * Function for part B H2
     * @param xi
     * @param eta
     * @return
     */
    public static double partBh2(Double xi, Double eta, Integer position) throws FunctionCalculationException {
        
        return Functions.partBh1(xi, eta, position);
    }
    
    /**
     * Function for part B q
     * @param xi
     * @param eta
     * @return
     */
    public static double partBq(Double xi, Double eta, Integer position) throws FunctionCalculationException {
        
        return 1.0;
    }
    
    /**
     * Function for part C H1
     * @param xi
     * @param eta
     * @return
     */
    public static double partCh1(Double xi, Double eta, Integer position) throws FunctionCalculationException {
        
        // Determine which position in the vector the user wants to calculate
        switch(position.intValue()) {
        
        	case 1:
        	    
        	    return (xi.doubleValue());
        	
        	case 2:

        	    return (eta.doubleValue());
        	
        	default:
                if(localLogger.isEnabledFor(Level.ERROR)) {
                    localLogger.log(Level.ERROR, "Error: Unknown position in function vector given !");
                }
                
                throw new FunctionCalculationException("Illegal position value !");
        }
    }
    
    /**
     * Function for part C H2
     * @param xi
     * @param eta
     * @return
     */
    public static double partCh2(Double xi, Double eta, Integer position) throws FunctionCalculationException {
        
        return Functions.partCh1(xi, eta, position);
    }
    
    /**
     * Function for part C q
     * @param xi
     * @param eta
     * @return
     */
    public static double partCq(Double xi, Double eta, Integer position) throws FunctionCalculationException {
        
        return (xi.doubleValue() * eta.doubleValue());
    }
    
    /**
     * Function for part D H1
     * @param xi
     * @param eta
     * @return
     */
    public static double partDh1(Double xi, Double eta, Integer position) throws FunctionCalculationException {
        
        // Determine which position in the vector the user wants to calculate
        switch(position.intValue()) {
        
        	case 1:
        	    
        	    return (1.0 - xi.doubleValue() - eta.doubleValue());
        	
        	case 2:

        	    return (xi.doubleValue());
        	
        	case 3:
        	    
        	    return (eta.doubleValue());
        	
        	default:
                if(localLogger.isEnabledFor(Level.ERROR)) {
                    localLogger.log(Level.ERROR, "Error: Unknown position in function vector given !");
                }
                
                throw new FunctionCalculationException("Illegal position value !");
        }
    }
    
    /**
     * Function for part D H2
     * @param xi
     * @param eta
     * @return
     */
    public static double partDh2(Double xi, Double eta, Integer position) throws FunctionCalculationException {
        
        return 1.0;
    }
    
    /**
     * Function for part D q
     * @param xi
     * @param eta
     * @return
     */
    public static double partDq(Double xi, Double eta, Integer position) throws FunctionCalculationException {
        
        return 1.0;
    }
    
    /**
     * Function for part E H1
     * @param xi
     * @param eta
     * @return
     */
    public static double partEh1(Double xi, Double eta, Integer position) throws FunctionCalculationException {
        
        // Determine which position in the vector the user wants to calculate
        switch(position.intValue()) {
        
        	case 1:
        	    
        	    return (-3+4*xi.doubleValue()+4*eta.doubleValue());
        	
        	case 2:

        	    return (-1+4*xi.doubleValue());
        	
        	case 3:
        	    
        	    return (0.0);
        	    
        	case 4:
        	    
        	    return (4-8*xi.doubleValue()-4*eta.doubleValue());
        	    
        	case 5:
        	    
        	    return (4*eta.doubleValue());
        	    
        	case 6:
        	    
        	    return (-4*eta.doubleValue());
        	    
        	default:
                if(localLogger.isEnabledFor(Level.ERROR)) {
                    localLogger.log(Level.ERROR, "Error: Unknown position in function vector given !");
                }
                
                throw new FunctionCalculationException("Illegal position value !");
        }
    }
    
    /**
     * Function for part E H2
     * @param xi
     * @param eta
     * @return
     */
    public static double partEh2(Double xi, Double eta, Integer position) throws FunctionCalculationException {
        
        // Determine which position in the vector the user wants to calculate
        switch(position.intValue()) {
        
        	case 1:
        	    
        	    return (-3+4*xi.doubleValue()+4*eta.doubleValue());
        	
        	case 2:

        	    return (0.0);
        	
        	case 3:
        	    
        	    return (-1+4*eta.doubleValue());
        	    
        	case 4:
        	    
        	    return (-4*xi.doubleValue());
        	    
        	case 5:
        	    
        	    return (4*xi.doubleValue());
        	    
        	case 6:
        	    
        	    return (4-4*xi.doubleValue()-8*eta.doubleValue());
        	    
        	default:
                if(localLogger.isEnabledFor(Level.ERROR)) {
                    localLogger.log(Level.ERROR, "Error: Unknown position in function vector given !");
                }
                
                throw new FunctionCalculationException("Illegal position value !");
        }
    }
    
    /**
     * Function for part E q
     * @param xi
     * @param eta
     * @return
     */
    public static double partEq(Double xi, Double eta, Integer position) throws FunctionCalculationException {
        
        return 1.0;
    }
    
    /**
     * Function for part F H1
     * @param xi
     * @param eta
     * @return
     */
    public static double partFh1(Double xi, Double eta, Integer position) throws FunctionCalculationException {
        
        // Determine which position in the vector the user wants to calculate
        switch(position.intValue()) {
        
        	case 1:
        	    
        	    return (0.0);
        	
        	case 2:

        	    return (-1*xi.doubleValue());
        	
        	case 3:
        	    
        	    return (xi.doubleValue());
        	
        	default:
                if(localLogger.isEnabledFor(Level.ERROR)) {
                    localLogger.log(Level.ERROR, "Error: Unknown position in function vector given !");
                }
                
                throw new FunctionCalculationException("Illegal position value !");
        }
    }
    
    /**
     * Function for part F H2
     * @param xi
     * @param eta
     * @return
     */
    public static double partFh2(Double xi, Double eta, Integer position) throws FunctionCalculationException {
        
        return Functions.partFh1(xi, eta, position);
    }
    
    /**
     * Function for part F q
     * @param xi
     * @param eta
     * @return
     */
    public static double partFq(Double xi, Double eta, Integer position) throws FunctionCalculationException {
        
        return 1.0/xi.doubleValue();
    }
}
