/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * DV-Anwendungen in der Technik - Prof.Dr.Gleich - 8.Semester
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 5.0
 * Semestergroup: 8T
 */

/**
 * USABILITY:
 *  Copyright (c) 2005 
 *  Thomas Woellert (thomas@woellert.net)
 *  Stefan Weissbach (weissbst@gmx.de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package fhm.edu.fem.calc.objects;

import java.lang.reflect.Method;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import fhm.edu.fem.util.logging.LoggingManager;

/**
 * Class Description:
 *		This class holds the methods to calculate the Fourier Coefficients
 *		via the double-integral and also a static method to calculate w0 at a given point (x,y)
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 * @author Stefan Weissbach (weissbst@gmx.de)
 */
public class MembraneFunctions {
	
	/* Our local logger object */
	protected static Logger localLogger = LoggingManager.getLogger(MembraneFunctions.class.getName());
	
	/* The method names - For figure Triangle - Linear approach */
	public static final String SIN_N_Y_B = "sinN";
	public static final String SIN_M_X_A  = "sinM";
	public static final String W0_X_Y = "w0";
	
	/* To statically remember the values we need for the calculation methods */
	protected static double outValueX;
	protected static double outValueY;
	protected static double outValueZ;
	protected static double dimensionA;
	protected static double dimensionB;
	
	/* Helper Method */
	public static final String ZERO = "zero";
	public static final String ONE = "one";
	
	/**
	 * Get the proper method
	 * @param methodName, the method name taken from the constants in this class
	 * @return the method
	 */
	public static Method getMethod(String methodName) throws SecurityException, NoSuchMethodException {
	    
	    // Build the args array
	    Class args[] = new Class[3];
	    args[0] = Double.class;
	    args[1] = Double.class;
	    args[2] = Integer.class;
	    
	    return MembraneFunctions.class.getDeclaredMethod(methodName, args);
	}
	
	/**
	 * To remember the membran's size and the "auslenkungs"-point
	 * @param midX, middle point X
	 * @param midY, middle point Y
	 * @param midZ, middle point Z
	 */
	public static void storeMembraneValues(double outX, double outY, double outZ, double dimA, double dimB) {
		
		outValueX = outX;
		outValueY = outY;
		outValueZ = outZ;
		dimensionA =dimA;
		dimensionB = dimB;
	}	
	
	/**
	 * Helper Method
	 * @param xi
	 * @param eta
	 * @param position
	 * @return
	 */
	public static double zero(Double xi, Double eta, Integer position) {
	    
	    return 0.0;
	}
	
	/**
	 * Helper Method
	 * @param xi
	 * @param eta
	 * @param position
	 * @return
	 */
	public static double one(Double xi, Double eta, Integer position) {
	    
	    return 1.0;
	}
    
	/**
	 * Static Method to calculate w(0)
	 * @param x, the x-value of the point to calculate w(0) for
	 * @param y, the y-value of the point to calculate w(0) for
	 * @param position, the position integer (not used in this method)
	 * @return double, the w(0) at the required point (x,y)
	 */
	public static double w0(Double x, Double y, Integer position) {

    		// Check in which Quadrant the node lies (I, II, III, IV)
			// y1 = "Gerade" from the lower-left corner to the mid-point
			// y2 = "Gerade" from the upper-left corner to the mid-point
			// y3 = "Gerade" from the upper-right corner to the mid-point
			// y4 = "Gerade" from the lower-right corner to the mid-point
		
    		//double y1 = ((outValueY - 0)/(outValueX - 0)) * x.doubleValue() + outValueY - ((outValueY - 0)/(outValueX - 0)) * outValueX;
    		//double y2 = ((outValueY - dimensionB)/(outValueX - 0)) * x.doubleValue() + outValueY - ((outValueY - dimensionB)/(outValueX - 0)) * outValueX;
    		//double y3 = ((outValueY - dimensionB)/(outValueX - dimensionA)) * x.doubleValue() + outValueY - ((outValueY - dimensionB)/(outValueX - dimensionA)) * outValueX;
    		//double y4 = ((outValueY - 0)/(outValueX - dimensionA)) * x.doubleValue() + outValueY - ((outValueY - 0)/(outValueX - dimensionA)) * outValueX;
    		
			double y1 = (outValueY/outValueX) * x.doubleValue();
			double y2 = ((outValueY-dimensionB)/outValueX) * x.doubleValue() + dimensionB;
			double y3 = ((dimensionB-outValueY)/(dimensionA-outValueX)) * (x.doubleValue() - dimensionA) + dimensionB;
			double y4 = (outValueY/(outValueX-dimensionA)) * (x.doubleValue() - dimensionA);
		
    		// To remember the calculated w0 value for this node
    		double w0Value = -1;
    		
    		// CHECK FOR REGION I
    		if((x.doubleValue() <= outValueX && y.doubleValue() <= y1) || (x.doubleValue() >= outValueX && y.doubleValue() <= y4)) {
    	        
    	        w0Value = (outValueZ / outValueY) * y.doubleValue();
    	        
    	        if(localLogger.isEnabledFor(Level.DEBUG)) {
    	            localLogger.log(Level.DEBUG, "x/y : " + x +"/" +y + " is part of region I");
    	            localLogger.log(Level.DEBUG, "w0:  " + w0Value);
    	            localLogger.log(Level.DEBUG, "Geraden Values " + y1 + " / " + y2 + " / " + y3 + " / " + y4);
    	        }
    	    
    	    // CHECK FOR REGION II
    		} else if(x.doubleValue() >= outValueX && y.doubleValue() <= y3 && y.doubleValue() >= y4) {
    			
    	        w0Value = (outValueZ / (outValueX - dimensionA)) * (x.doubleValue() - dimensionA);
    	        
    	        if(localLogger.isEnabledFor(Level.DEBUG)) {
    	            localLogger.log(Level.DEBUG, "x/y : " + x +"/" +y + " is part of region II");
    	            localLogger.log(Level.DEBUG, "w0:  " + w0Value);
    	            localLogger.log(Level.DEBUG, "Geraden Values " + y1 + " / " + y2 + " / " + y3 + " / " + y4);
    	        }
    	        
    	    // CHECK FOR REGION III
    		} else if((x.doubleValue() <= outValueX && y.doubleValue() >= y2) || (x.doubleValue() >= outValueX && y.doubleValue() >= y3)) {
    			
    	        w0Value = (outValueZ / (outValueY - dimensionB)) * (y.doubleValue() - dimensionB); 
    	        
    	        if(localLogger.isEnabledFor(Level.DEBUG)) {
    	            localLogger.log(Level.DEBUG, "x/y : " + x +"/" +y + " is part of region III");
    	            localLogger.log(Level.DEBUG, "w0:  " + w0Value);
    	            localLogger.log(Level.DEBUG, "Geraden Values " + y1 + " / " + y2 + " / " + y3 + " / " + y4);
    	        }
    	        
    	    // CHECK FOR REGION IV
    		} else if(x.doubleValue() <= outValueX && y.doubleValue() >= y1 && y.doubleValue() <= y2) {
    			
    	        w0Value = (outValueZ/outValueX) * x.doubleValue();
    	        
    	        if(localLogger.isEnabledFor(Level.DEBUG)) {
    	            localLogger.log(Level.DEBUG, "x/y : " + x +"/" +y + " is part of region IV");
    	            localLogger.log(Level.DEBUG, "w0:  " + w0Value);
    	            localLogger.log(Level.DEBUG, "Geraden Values " + y1 + " / " + y2 + " / " + y3 + " / " + y4);
    	        }
    			
    		} else {
    			
    	        if(localLogger.isEnabledFor(Level.ERROR)) {
    	            localLogger.log(Level.ERROR, "x/y : " + x +"/" +y + " is not part of the Membran?!?");
    	            localLogger.log(Level.ERROR, "Midpoint " + outValueX + " / " + outValueY);
    	            localLogger.log(Level.ERROR, "Geraden Values " + y1 + " / " +y2);
    	        }
    	        
    	        return Double.MIN_VALUE;
    		}

    		// Return the result
    		return w0Value;
	}
	
	/**
	 * Calculate sin(m*pi*(x/a)
	 * @param x, the x value
	 * @param y, the y value (not used in this method)
	 * @param m, the integer m
	 * @return the result
	 */
	public static double sinM(Double x, Double y, Integer m) {

	    return Math.sin(m.intValue() * (Math.PI) * (x.doubleValue() / dimensionA));
	}
	
	/**
	 * Calculate sin(n*pi*(y/b)
	 * @param x, the x value (not used in this method)
	 * @param y, the y value
	 * @param m, the integer m
	 * @return the result
	 */
	public static double sinN(Double x, Double y, Integer n) {

	    return Math.sin(n.intValue() * (Math.PI) * (y.doubleValue() / dimensionB));
	}
}
