/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * DV-Anwendungen in der Technik - Prof.Dr.Gleich - 8.Semester
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 5.0
 * Semestergroup: 8T
 */

/**
 * USABILITY:
 *  Copyright (c) 2005 
 *  Thomas Woellert (thomas@woellert.net)
 *  Stefan Weissbach (weissbst@gmx.de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package fhm.edu.fem.calc.objects;

/**
 * Class Description:
 *		This class acts as "node"
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 * @author Stefan Weissbach (weissbst@gmx.de)
 */
public class Node {

    /* The id identifying this node */
    protected int id = -1;
    
    /* The id identifying this node - Membrane Calculation - Alias ID */
    protected int aliasId = 0;
    
    /* The x-value of this node */
    protected double x = -1;
    
    /* The y-value of this node */
    protected double y = -1;
    
    /* Boolean telling us if this is a PSEUDO node or not */
    protected boolean isPseudoNode = false;
    
    /* The SPANNER original X-value */
    protected double xOriginal = -1;
    
    /* The SPANNER original Y-value */
    protected double yOriginal = -1;
    
    /**
     * Construct a new NON-PSEUDO node with the given ID
     * @param id, the ID for this node
     * @param x, the x coordinate for this node
     * @param y, the y coordinate for this node
     * @param isPseudoNode, true if it is, false otherwise
     */
    public Node(int id, double x, double y, boolean isPseudoNode) {
        this.id = id;
        this.x = x;
        this.y = y;
        this.xOriginal = x;
        this.yOriginal = y;
        this.isPseudoNode = isPseudoNode;
    }
    
    /**
     * @return Returns the id.
     */
    public int getId() {
        return id;
    }
    
    /**
     * @return Returns the alias id.
     */
    public int getAliasId() {
    	return aliasId;
    }
    
    /**
     * Set the alias Id
     * @param newId, the new alias Id
     */
    public void setAliasId(int newId) {
    	aliasId = newId;
    }
    
    /**
     * @return Returns the x.
     */
    public double getX() {
        return x;
    }
    
    /**
     * @return Returns the y.
     */
    public double getY() {
        return y;
    }
    
    /**
     * @return Returns the isPseudoNode.
     */
    public boolean isPseudoNode() {
        return isPseudoNode;
    }
    
    /**
     * Implementation of the toString() method
     * @return a string visualization of the node
     */
    public String toString() {
        
        return "[Node id=" + this.id + " aliasId = " + this.aliasId + " x=" + this.x + " y=" + this.y + "]";
    }
    
    /**
     * Compare this node with another given node
     * @param obj, the other node
     */
    public boolean equals(Object obj) {
        
        // Check if the obj is an instance of Node
        if(!(obj instanceof Node)) {
            
            return false;
        }
        
        // We got a node, compare it
        Node node = (Node)(obj);
        
        // Compare everything
        if(this.id != node.getId()) {
            
            return false;
            
        } else if(this.x != node.getX()) {
            
            return false;
            
        } else if(this.y != node.getY()) {
            
            return false;
            
        } else if(this.isPseudoNode != node.isPseudoNode()) {
            
            return false;
        }
        
        return true;
    }
    
	/**
	 * Clone Implementation
	 * @return a clone of this list
	 */
	public Object clone() {
		
		// Create the new node
		Node cloneNode = new Node(this.id, this.x, this.y, this.isPseudoNode);
		cloneNode.setAliasId(this.aliasId);
				
		return cloneNode;
	}
	
	/**
	 * @param x The x to set.
	 */
	public void setX(double x) {
		this.x = x;
	}
	
	/**
	 * @param y The y to set.
	 */
	public void setY(double y) {
		this.y = y;
	}
	
	/**
	 * @return Returns the xOriginal.
	 */
	public double getXOriginal() {
		return xOriginal;
	}
	
	/**
	 * @param original The xOriginal to set.
	 */
	public void setXOriginal(double original) {
		xOriginal = original;
	}
	
	/**
	 * @return Returns the yOriginal.
	 */
	public double getYOriginal() {
		return yOriginal;
	}
	
	/**
	 * @param original The yOriginal to set.
	 */
	public void setYOriginal(double original) {
		yOriginal = original;
	}
}
