/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * DV-Anwendungen in der Technik - Prof.Dr.Gleich - 8.Semester
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 5.0
 * Semestergroup: 8T
 */

/**
 * USABILITY:
 *  Copyright (c) 2005 
 *  Thomas Woellert (thomas@woellert.net)
 *  Stefan Weissbach (weissbst@gmx.de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package fhm.edu.fem.calc.objects.matrix;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import fhm.edu.fem.constants.ApplicationConstants;
import fhm.edu.fem.util.logging.LoggingManager;

/**
 * Class Description:
 *		This class creates the different types of matrices
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 * @author Stefan Weissbach (weissbst@gmx.de)
 */
public class MatrixFactory {
    
	/* Our local logger object */
	protected static Logger localLogger = LoggingManager.getLogger(MatrixFactory.class.getName());
    
	/** 
	 * To make sure noone is using this constructor.
	 * Else the compiler would create a public one.
	 * */
	private MatrixFactory() {}
	
	/**
	 * Create a new Standard Matrix
	 * @param name, the name of the matrix
	 * @param size, the size of the matrix (symmetric)
	 * @return the created matrix
	 */
	public static MatrixStandard createStandardMatrix(String name, int size) {

	    return new MatrixStandard(name, size);
	}
	
	/**
	 * Create a new Standard Matrix
	 * @param name, the name of the matrix
	 * @param values, double array holding the values to store
	 * @return the created matrix
	 */
	public static MatrixStandard createStandardMatrix(String name, double[][] values) {

	    return new MatrixStandard(name, values);
	}
	
	/**
	 * Create a new Hull Matrix
	 * @param name, the name of the matrix
	 * @param size, the size of the matrix (symmetric)
	 * @return the created matrix
	 */
	public static MatrixHull createHullMatrix(String name, int size) {

	    return new MatrixHull(name, size);
	}
	
	/**
	 * Create a new Hull Matrix
	 * @param name, the name of the matrix
	 * @param values, double array holding the values to store
	 * @return the created matrix
	 */
	public static MatrixHull createHullMatrix(String name, double[][] values) {

	    return new MatrixHull(name, values);
	}
	
	/**
	 * Creates the type of matrix the user defined in the constants file
	 * @param name, the name of the matrix
	 * @param size, the size of the matrix (symmetric)
	 * @return the created matrix
	 */
	public static Matrix createUserDefinedMatrix(String name, int size) {
	    
	    // Check which kind of matrix the user wants
	    if(ApplicationConstants.MATRIX_USE == ApplicationConstants.MATRIX_STANDARD) {
	        
	        // Create a standard matrix
	        return MatrixFactory.createStandardMatrix(name, size);
	        
	    } else if(ApplicationConstants.MATRIX_USE == ApplicationConstants.MATRIX_HULL) {
	        
	        // Create a hull matrix
	        return MatrixFactory.createHullMatrix(name, size);
	        
	    } else {
	        
	        // Error case, give the user an error message and create a standard matrix
            if(localLogger.isEnabledFor(Level.ERROR)) {
                localLogger.log(Level.ERROR, "Error: Unknown matrix type defined in ApplicationConstants: " + ApplicationConstants.MATRIX_USE + " creating standard matrix object !");
            }
	        
	        return MatrixFactory.createStandardMatrix(name, size);
	    }
	}
	
	/**
	 * Creates the type of matrix the user defined in the constants file
	 * @param name, the name of the matrix
	 * @param values, double array holding the values (has to be quadratic)
	 * @return the created matrix
	 */
	public static Matrix createUserDefinedMatrix(String name, double[][] values) {
	    
	    // Check which kind of matrix the user wants
	    if(ApplicationConstants.MATRIX_USE == ApplicationConstants.MATRIX_STANDARD) {
	        
	        // Create a standard matrix
	        return MatrixFactory.createStandardMatrix(name, values);
	        
	    } else if(ApplicationConstants.MATRIX_USE == ApplicationConstants.MATRIX_HULL) {
	        
	        // Create a hull matrix
	        return MatrixFactory.createHullMatrix(name, values);
	        
	    } else {
	        
	        // Error case, give the user an error message and create a standard matrix
            if(localLogger.isEnabledFor(Level.ERROR)) {
                localLogger.log(Level.ERROR, "Error: Unknown matrix type defined in ApplicationConstants: " + ApplicationConstants.MATRIX_USE + " creating standard matrix object !");
            }
	        
	        return MatrixFactory.createStandardMatrix(name, values);
	    }
	}
}
