/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * DV-Anwendungen in der Technik - Prof.Dr.Gleich - 8.Semester
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 5.0
 * Semestergroup: 8T
 */

/**
 * USABILITY:
 *  Copyright (c) 2005 
 *  Thomas Woellert (thomas@woellert.net)
 *  Stefan Weissbach (weissbst@gmx.de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package fhm.edu.fem.calc.objects.matrix;

import org.apache.log4j.Level;

import fhm.edu.fem.constants.ApplicationConstants;
import fhm.edu.fem.exception.CholeskyFailedException;
import fhm.edu.fem.test.ENVHullChainMatrix;

/**
 * Class Description:
 *		Test implementation of a Version of the Hull Matrix which is faster
 *		and does not have the "small" speed problems of the original MatrixHull class.
 *		
 *		ATTENTION:
 *		THIS TEST IMPLEMENTATION IS NOT WORKING AND NOT COMPLETE SO DO NOT
 *		USE IT IN A WORKING ENVIRONMENT.
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 * @author Stefan Weissbach (weissbst@gmx.de)
 */
public class MatrixHullChain extends MatrixHull {
    
    /**
     * Create a new HullMatrix with the size (x rows and x columns) (all fields are initialized with zeros)
     * @param name, the name of the matrix
     * @param size, the number of rows and columns (square matrix only!)
     */
    public MatrixHullChain(String name, int size) {

        this.name = name;
        this.size = size;

        // Create the diagonal-pointer-vector
        this.diagPointer = new int[size+1];
        
        // Create the array holding the data
        this.env = new ENVHullChainMatrix(size);
        
        // Initialize the diagonal-pointer-vector
        for(int i = 0; i <= size; i++) {

            this.diagPointer[i] = i;
        }
    }
    
    /**
     * Copy Constructor
     * @param m, the hull matrix to copy
     */
    public MatrixHullChain(MatrixHullChain m) {
        
        // Retrieve the name and size of the matrix to copy
        this.name = m.getName();
        this.size = m.size();
        
        // Retrieve the diag pointer of the other matrix and clone it, making it our own        
        this.diagPointer = new int[m.getDiagPointer().length];
        
        for(int i = 0; i < this.diagPointer.length; i++) {
            this.diagPointer[i] = m.getDiagPointer()[i];
        }
        
        // Retrieve the ENV object from the other matrix and make a copy
        this.env = (ENVHullChainMatrix)(m.getENV().clone());
    }
    
    /**
     * Do the cholesky for this matrix
     * @return the matrix L as result of the cholesky
     */
    public Matrix cholesky() throws CholeskyFailedException {
        
        // Create the new Matrix L we are going to create
        MatrixHull L = new MatrixHull("Cholesky L for Matrix " + this.name, this.size); 
                
        // Calculate how many percentage one line makes up
        double oneLinePercentage = 100.0 / this.size;
        
        // Calculate how many lines make up one percent
        double onePercentLines = this.size / 100.0;
        
        // Where we store the current actual percentage
        double currentPercentage = 0.0;
        
        // Initialize variables here to save allocation time when running
        int startIndexForK = -1;
        int startIndexForJ = -1;
        
        // Run through all rows in the matrix
        for(int k = 1; k <= this.size; k++) {

            // Calculate the current percentage of line already done in the CHOLESKY
            currentPercentage = (k-1) * oneLinePercentage;

            // Make a percentage output about every 2% of completed lines
            if(k%(onePercentLines*2.0) < 1) {
                
//                if(localLogger.isEnabledFor(Level.INFO)) {
//                    localLogger.log(Level.INFO, "Cholesky at " + currentPercentage + " %");
//                }
            }

            // Get the value from the matrix and check if the value is greater than 0 (a MUST HAVE)
            double valueA = this.getValue(k, k);
            
            if(valueA <= 0) {
                
                if(localLogger.isEnabledFor(Level.ERROR)) {
                    localLogger.log(Level.ERROR, "Error while computing the CHOLESKY: Main Diagonal Element is negative at index: " + k);
                }
                
                throw new CholeskyFailedException();
            }
            
            // Calculate the sum for the square root (Sum from i=1 till (k-1) l[k][i]^2)
            double sum = 0;
            
            // PERFORMANCE: Check where the first column starts, which we can find out at this point to save calculation time
            startIndexForK = this.getFirstNonZeroIndexForRow(k);
            
            int iCount = startIndexForK;
            
            while(iCount <= (k-1)) {

                sum = sum + Math.pow(L.getValue(k, iCount), 2);
                
                // PERFORMANCE: Determine the next i-index by checking for the next non-zero value from the current one
                iCount = this.getNextNonZeroIndexForRow(k, iCount);

                // Check if a next-value has not been found
                if(iCount == -1) {
                    
                    // No non-zero value in that row anymore, so go to the next one
                    break;
                }
            }

            double valueL = Math.sqrt(valueA - sum);

            // Set the new L value at l[k][k]
            L.setValue(k, k, valueL);

            // Calculate l[j][k] = (1/l[k][k]) * (a[j][k] - sum[from i=1 till k-1](l[j][i] * l[k][i])
            for(int j = (k+1); j <= this.size; j++) {

                // Reset the sum and calculate it
                sum = 0;
                
                // PERFORMANCE: Check where the first column starts, which we can find out at this point to save calculation time
                startIndexForJ = this.getFirstNonZeroIndexForRow(j);
                
                // PERFORMANCE: Determine which one is the maximum index because we need both in a unified for-loop               
                 for(int i = Math.max(startIndexForK, startIndexForJ); i <= (k-1); i++) {
                     sum = sum + (L.getValue(j, i) * L.getValue(k, i));
                }
                
                // Calculate the new L-value
                double newValueL = (this.getValue(j, k) - sum) / valueL;
                
                // Set l[j][k]
                L.setValue(j, k, newValueL);
            }
        }

        return L;
    }
    
    /**
     * Clone the matrix
     * @return the cloned matrix
     */
     public Object clone() {
         
         // Make a clone matrix using the copy constructor
         MatrixHull clone = new MatrixHullChain(this);
         
         // Return the cloned matrix
         return clone;
     }
     
     /* (non-Javadoc)
      * @see java.lang.Object#toString()
      */
     public String toString() 
     {
         // Create a new empty string
         String stringMatrix = "[MatrixHullChain - \nName: " + this.name + "\nSize: " + this.size + "\nData: \n";
         
         // Build a string-matrix to display
         for(int i = 1; i <= this.size(); i++)
         {
             for(int j = 1; j <= this.size(); j++)
             {
                 stringMatrix = stringMatrix + this.getValue(i, j) + " \t ";
             }
             stringMatrix = stringMatrix + '\n';
         }
         
         return stringMatrix + "]";
     } 

     /**
      * Retrieve the amount of required memory in this matrix (in byte)
      * @return the amount of required memory in byte
      */
     public long getRequiredMemory() {
         
         long bytes = 0;
         
         // Add the size-integer to the required amount of bytes
         bytes += ApplicationConstants.SIZE_INT;
         
         // Add the one-dimensional diag-pointer to the required amount of bytes
         bytes += ApplicationConstants.SIZE_INT * this.size;
         
         // Add the required memory of the ENVHullMatrix to the required amount of bytes
         bytes += this.env.getRequiredMemory();
         
         // Add the string-name to the required amount of bytes
         bytes += ApplicationConstants.SIZE_CHAR * this.name.length();
         
         return bytes;
     }
     
     /**
      * Find the next non-zero value starting from the given column index in the given row
      * @param row
      * @param column
      * @return
      * @throws IndexOutOfBoundsException
      */
     protected int getNextNonZeroIndexForRow(int row, int column) throws IndexOutOfBoundsException {

         // Check if we got a proper index
         if(row > this.size || column > this.size || row <= 0 || column <= 0)
         {
             throw new IndexOutOfBoundsException();
         }
         
         // Calculate the index of the current item in the ENV
         
         // All matrices we work with here are symmetric
         if(column > row) {
             return this.getNextNonZeroIndexForRow(column, row);
         }
                 
         // We got a proper index for both row and column and are also lumping around in the lower triangle of the matrix
         
         // Calculate the hull-element-index where the element is located (no matter if we have it or not)
         int envIndex = this.diagPointer[row] - (row - column);

         // Calculate the row-end-index telling the ENV how far to search for the next non-zero item
         int envLastRowIndex = -1;
         
         if(row < this.diagPointer.length) {
             
             envLastRowIndex = this.diagPointer[row];
             
         } else {
             
             envLastRowIndex = this.env.size();
         }
         
         // Call the proper method in the ENV
         int envNextItemIndex = ((ENVHullChainMatrix)(this.env)).getNextNonZeroIndex(envIndex, envLastRowIndex);

         // Recalculate the found index to convert it into the normal matrix index if necessary
         if(envNextItemIndex != -1) {
             
             return envNextItemIndex + row - this.diagPointer[row];
             
         } else {
             
             return envNextItemIndex;
         }
     }
     
     public void displayChain() {
         System.err.println(((ENVHullChainMatrix)(this.env)).displayChain());
     }
}
