/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * DV-Anwendungen in der Technik - Prof.Dr.Gleich - 8.Semester
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 5.0
 * Semestergroup: 8T
 */

/**
 * USABILITY:
 *  Copyright (c) 2005 
 *  Thomas Woellert (thomas@woellert.net)
 *  Stefan Weissbach (weissbst@gmx.de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package fhm.edu.fem.calc.objects.matrix;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.LayoutManager;
import java.awt.geom.AffineTransform;
import java.awt.image.BufferedImage;

import javax.swing.JPanel;

/**
 * Class Description:
 *		This class is an extension of JPanel to make a Matrix
 *		displayable to the user
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 * @author Stefan Weissbach (weissbst@gmx.de)
 */
public class MatrixPanel extends JPanel {

    // The Matrix which we paint
    protected Matrix matrix = null;
    
    // The Background color
    protected Color backgroundColor = Color.WHITE;
    
    // The Draw color
    protected Color drawColor = Color.BLACK;
    
    // The image we draw into
    protected BufferedImage matrixImage = null; 
    
    /**
     * Create a new MatrixPanel
     * @param matrix, the matrix to display
     */
    public MatrixPanel(Matrix matrix) {
        super();
        
        this.matrix = matrix;
        this.setBackground(backgroundColor);
        this.createImage();
    }
        
    /**
     * Create a new MatrixPanel
     * @param matrix, the matrix to display
     * @param isDoubleBuffered
     */
    public MatrixPanel(Matrix matrix, boolean isDoubleBuffered) {
        super(isDoubleBuffered);
        
        this.matrix = matrix;
        this.setBackground(backgroundColor);
        this.createImage();
    }
    
    /**
     * Create a new MatrixPanel
     * @param matrix, the matrix to display
     * @param layout
     */
    public MatrixPanel(Matrix matrix, LayoutManager layout) {
        super(layout);
        
        this.matrix = matrix;
        this.setBackground(backgroundColor);
        this.createImage();
    }
    
    /**
     * Create a new MatrixPanel
     * @param matrix, the matrix to display
     * @param layout
     * @param isDoubleBuffered
     */
    public MatrixPanel(Matrix matrix, LayoutManager layout, boolean isDoubleBuffered) {
        super(layout, isDoubleBuffered);
        
        this.matrix = matrix;
        this.setBackground(backgroundColor);
        this.createImage();
    }    
    
    /**
     * Paint the Panel
     * @param g, the graphics we are allowed to draw into
     */
    protected void paintComponent(Graphics g) {
        
        // Call the super paint method of the JPanel
        super.paintComponent(g);
        
        // Create the graphics2D object
        Graphics2D g2 = (Graphics2D)g;
        AffineTransform transform = new AffineTransform();
        
        // Check if we need to scale the image
        if(this.getHeight() != this.matrix.size() || this.getWidth() != this.matrix.size()) {
            
            // We need to apply the affine transform, calculate the scale factors
            double xScale = (double)(this.getWidth()) / (double)(this.matrix.size());
            double yScale = (double)(this.getHeight()) / (double)(this.matrix.size());
            
            // Apply the scale factors
            transform.scale(xScale, yScale);           
        }
        
        g2.setTransform(transform); 
        g2.drawImage(this.matrixImage, 0, 0, this);
    }
    
    /**
     * Create and draw the image
     */
    protected void createImage() {
        
        // Create the image
        int matrixSize = this.matrix.size();
        this.matrixImage = new BufferedImage(matrixSize, matrixSize, BufferedImage.TYPE_3BYTE_BGR);
        Graphics2D g2 = this.matrixImage.createGraphics();
        
        // Clear the background of the graphic
        g2.setPaint(this.backgroundColor);
        g2.fillRect(0, 0, matrixSize, matrixSize);
        
        // Draw the matrix
        g2.setPaint(this.drawColor);
        
        for(int row = 1; row <= matrixSize; row++) {
            for(int col = 1; col <= matrixSize; col++) {
                
                // Check if there is an item at this location
                if(this.matrix.getValue(row, col) != 0) {

                    // Draw the current point representing a value != 0
                    g2.fillRect(row, col, 1, 1);
                }
            }
        }
    }
}
