/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * DV-Anwendungen in der Technik - Prof.Dr.Gleich - 8.Semester
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 5.0
 * Semestergroup: 8T
 */

/**
 * USABILITY:
 *  Copyright (c) 2005 
 *  Thomas Woellert (thomas@woellert.net)
 *  Stefan Weissbach (weissbst@gmx.de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package fhm.edu.fem.calc.objects.matrix;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import fhm.edu.fem.calc.objects.Vector;
import fhm.edu.fem.constants.ApplicationConstants;
import fhm.edu.fem.exception.CholeskyFailedException;
import fhm.edu.fem.util.logging.LoggingManager;

/**
 * Class Description:
 *		This class represents the matrix (we start at index 1 not 0 for both the rows and the columns)
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 * @author Stefan Weissbach (weissbst@gmx.de)
 */
public class MatrixStandard implements Matrix {
    
	/* Our local logger object */
	protected static Logger localLogger = LoggingManager.getLogger(MatrixStandard.class.getName());
    
    // The rows and columns in the matrix
    protected int size;
    
    // The matrix itself
    protected double[][] data = null;
    
    // The name of the matrix, if it has one
    protected String name = "";
    
    /**
     * Matrix with the given name, filled with the given values
     * @param name, name
     * @param values, quadratic
     */
    public MatrixStandard(String name, double[][] values) {
    	
    	this.name = name;
        this.size = values.length;
        this.data = new double[this.size][this.size];
        
        // Store the values
        for(int i = 0; i < values.length; i++) {
        	for(int j = 0; j < values.length; j++) {
        		
        		data[i][j] = values[i][j];
        	}
        }
    }
    
    /**
     * Matrix with the size (x rows and x columns) (all fields are initialized with zeros)
     * @param name, the name of the matrix
     * @param size, the number of rows and columns (square matrix only!)
     */
    public MatrixStandard(String name, int size) 
    {
        this.name = name;
        this.size = size;
        this.data = new double[size][size];
    }
    
    /**
     * Retrieve a value from the matrix
     * @param row, the row index
     * @param colum, the column index
     * @return the value
     * @throws IndexOutOfBoundsException
     */
    public double getValue(int row, int column) throws IndexOutOfBoundsException
    {
        // Check if we got a proper index
        if(row > this.size || column > this.size || row <= 0 || column <= 0)
        {
            throw new IndexOutOfBoundsException();
        }
        
        // Decrement the row and column index by one because we start at row/column index 1        
        return this.data[row-1][column-1];
    }
    
    /**
     * Set a new value in the matrix
     * @param row, the row index
     * @param colum, the column index
     * @param value, the value to set
     * @throws IndexOutOfBoundsException
     */
    public void setValue(int row, int column, double value) throws IndexOutOfBoundsException
    {
        // Check if we got a proper index
        if(row > this.size || column > this.size || row <= 0 || column <= 0)
	    {
            throw new IndexOutOfBoundsException();
	    }
	
        // Decrement the row and column index by one because we start at row/column index 1
	    this.data[row-1][column-1] = value;
	}
    
    /**
     * Store a whole line in the matrix at the given line number
     * @param lineNumber, where to store the line
     * @param line, the array holding the line's data
     */
    public void storeLine(int lineNumber,  double[] line)
    {
        // Run through the array and store each value in the given line
        for(int i = 1; i <= line.length; i++){      
            
            // Set the proper value, we need to add 1 to the index i because our matrix index is starting at 1 not 0
            this.setValue(lineNumber, i, line[i-1]);
        }
    }
    
    /**
     * Set a whole column to the given value in the matrix at the given column number
     * @param columnNumber, where to store the column
     * @param value, the sole value we want to fill the column with
     */
    public void setLineAndColumn(int index, double value) {
        
        // Run through the specified line of the matrix and store the given value
        for(int i = 1; i <= this.size; i++) {
            this.setValue(index, i, value);
        }
        
        // Run through the specified column of the matrix and store the given value
        for(int i = 1; i <= this.size; i++) {
            this.setValue(i, index, value);
        }
    }
    
    /**
     * Get the number of rows and columns in the matrix (square matrix only!)
     * @return the number of rows and columns
     */
    public int size() {
      return this.size;  
    }
    
   /* (non-Javadoc)
    * @see java.lang.Object#toString()
    */
   public String toString() 
   {
       // Create a new empty string
       String stringMatrix = "[MatrixStandard - \nName: " + this.name + "\nSize: " + this.size + "\nData: \n";
       
       // Build a string-matrix to display
       for(int i = 1; i <= this.size(); i++)
       {
           for(int j = 1; j <= this.size(); j++)
           {
               stringMatrix = stringMatrix + this.getValue(i, j) + " \t ";
           }
           stringMatrix = stringMatrix + '\n';
       }
       
       return stringMatrix + "]";
   } 
   
   /**
    * Variable toString method
    * @param number, the number of rows and columns to display (starting at the upper left)
    * @return String
    */
   public String toString(int number) 
   {
       // Create a new empty string
       String stringMatrix = "[MatrixStandard - \nName: " + this.name + "\nSize: " + this.size + "\nData: \n";
       
       // Check if the required display size is bigger than the actual matrix size
       if(number > this.size()) {
       		// Reset the number to our current size
       		number = this.size();
       }
       
       // Build a string-matrix to display
       for(int i = 1; i <= number; i++)
       {
           for(int j = 1; j <= number; j++)
           {
               stringMatrix = stringMatrix + this.getValue(i, j) + " \t ";
           }
           stringMatrix = stringMatrix + '\n';
       }
       
       return stringMatrix + "]";
   } 
   
   /**
    * Multiply all values in the matrix by the given value
    * @param multiplicator, the value to multiply the matrix with
    * @return the new matrix, the original matrix is not changed
    */
   public Matrix mult(double multiplicator) {

       // The resulting matrix
       MatrixStandard result = new MatrixStandard(this.name, this.size());
       
       // Run through all rows and columns in the matrix
       for(int i = 1 ; i <= this.size(); i++)
       {
           for(int j = 1; j <= this.size(); j++)
           {
               result.setValue(i, j, this.getValue(i, j) * multiplicator);
           }
       }
       
       // Return the new matrix
       return result;
   }
   
   /**
    * Multiply this Matrix with the given vector (have to be the same size)
    * @param m, the vector
    * @return Vector, the result vector
    */
   public Vector mult(Vector m) {
   	
   	// Create the result vector
   	Vector result = new Vector("Result of Matrix * Vector", this.size());
   	
   	// Check the sizes
   	if(m.size() != this.size()) {
   		
           if(localLogger.isEnabledFor(Level.ERROR)) {
               localLogger.log(Level.ERROR, "Cannot multiply Matrix with the given Vector, the sizes do not match !");
           }
           
           return result;
   	}
   	
   	for(int i = 1; i <= this.size(); i++) {
   		
   		double resultValue = 0;
   		
   		for(int j = 1; j <= this.size(); j++) {
   			
   			resultValue += m.getValue(j) * this.getValue(i, j);
   		}
   		
   		result.setValue(i, resultValue);
   	}
   	
   	return result;
   }
   
   /**
    * Add a given matrix to this matrix
    * @param m, the matrix to add
    * @return the new matrix, the original matrix is not changed
    */
   public Matrix add(Matrix m) {
       
       // Check if both matrices have the same size
       if(this.size() != m.size()) {
           
           if(localLogger.isEnabledFor(Level.ERROR)) {
               localLogger.log(Level.ERROR, "Adding one matrix to another failed ! Not the same size !");
           }
           
           return null;
       }
       
       // Create a new Matrix holding the result
       MatrixStandard result = new MatrixStandard(this.name, this.size());
       
       // Run through all rows and columns in the matrix
       for(int i = 1 ; i <= this.size(); i++)
       {
           for(int j = 1; j <= this.size(); j++)
           {
               result.setValue(i, j, this.getValue(i, j) + m.getValue(i, j));
           }
       }
       
       // Return the resulting matrix
       return result;       
   }
   
   /**
    * Test if the Matrix is symmetric
    * @return false if it is not symmetric, true otherwise
    */
   public boolean isSymmetric() {
       
       // Check the values, run through all rows and values in the matrix
       for(int i = 1; i <= this.size; i++) {
           for(int j = 1; j <= i; j++) {
               
               // Compare the value
               if(this.getValue(i, j) != this.getValue(j, i)) {
                   
                   // The values are not the same, so the matrix is not symmetric
                   return false;
               }
           }
       }
       
       // All values where the same, the matrix is symmetric
       return true;
   }
   
   /**
    * Test if the main diagonal of this matrix is not zero
    * @return true if it is not zero, false otherwise
    */
   public boolean isMainDiagonalNotZero() {

       // Check if the main diagonal is not zero
       for(int i = 1; i <= this.size; i++) {
           
           if(this.getValue(i, i) == 0) {
               
               // The main diagonal element is zero
               return false;
           }
       }
       
       // No main diagonal element has been zero
       return true;
   }

   /**
    * Clone the matrix
    * @return the cloned matrix
    */
    public Object clone() {
        
        // Make a clone matrix
        MatrixStandard clone = new MatrixStandard(this.name, this.size);
        
        // Run through all rows and columns of the matrix
        for(int i = 1 ; i <= this.size(); i++)
        {
            for(int j = 1; j <= this.size(); j++)
            {
                clone.setValue(i, j, this.getValue(i, j));
            }
        }
        
        // Return the cloned matrix
        return clone;
    }
    
    /**
     * Do the cholesky for this matrix
     * @return the matrix L as result of the cholesky
     */
    public Matrix cholesky() throws CholeskyFailedException {
        
        // Create the new Matrix L we are going to create
        MatrixStandard L = new MatrixStandard("Cholesky L for Matrix " + this.name, this.size);
        
        // Calculate how many percentage one line makes up
        double oneLinePercentage = 100.0 / this.size;
        
        // Calculate how many lines make up one percent
        double onePercentLines = this.size / 100.0;
        
        // Where we store the current actual percentage
        double currentPercentage = 0.0;
        
        // Run through all rows in the matrix
        for(int k = 1; k <= this.size; k++) {
            
            // Calculate the current percentage of line already done in the CHOLESKY
            currentPercentage = (k-1) * oneLinePercentage;

            // Make a percentage output about every 2% of completed lines
            if(k%(onePercentLines*2.0) < 1) {
                
                if(localLogger.isEnabledFor(Level.INFO)) {
                    localLogger.log(Level.INFO, "Cholesky at " + ApplicationConstants.convertPercentageToString(currentPercentage) + " %");
                }
            }
            
            // Get the value from the matrix and check if the value is greater than 0 (a MUST HAVE)
            double valueA = this.getValue(k, k);
            
            if(valueA <= 0) {
                
                if(localLogger.isEnabledFor(Level.ERROR)) {
                    localLogger.log(Level.ERROR, "Error while computing the CHOLESKY: Main Diagonal Element is negative at index: " + k);
                }
                
                throw new CholeskyFailedException();
            }
            
            // Calculate the sum for the square root (Sum from i=1 till (k-1) l[k][i]^2)
            double sum = 0;
            
            for(int i = 1; i <= (k-1); i++) {
                sum = sum + Math.pow(L.getValue(k, i), 2);
            }
            
            double valueL = Math.sqrt(valueA - sum);
            
            // Set the new L value at l[k][k]
            L.setValue(k, k, valueL);
            
            // Calculate l[j][k] = (1/l[k][k]) * (a[j][k] - sum[from i=1 till k-1](l[j][i] * l[k][i])
            for(int j = (k+1); j <= this.size; j++) {
                
                // Reset the sum and calculate it
                sum = 0;
                
                for(int i = 1; i <= (k-1); i++) {
                    sum = sum + (L.getValue(j, i) * L.getValue(k, i));
                }
                
                // Calculate the new L-value
                double newValueL = (this.getValue(j, k) - sum) / valueL;
                
                // Set l[j][k]
                L.setValue(j, k, newValueL);
            }
        }
        
        return L;
    }
    
    /**
     * Transponate the Matrix
     * @return the new transponated Matrix
     */
    public Matrix transponate() {
    	
    	// Create the new Matrix
    	MatrixStandard result = new MatrixStandard(this.name + " transponated", this.size());
    	
    	for(int i = 1; i <= result.size(); i++) {
    		for(int j = 1; j <= result.size(); j++) {
    			
    			result.setValue(i, j, this.getValue(j, i));
    		}
    	}
    	
    	return result;
    }
    
    /**
     * Multiply this Matrix with the given Matrix (have to be the same size)
     * @param m, the matrix
     * @return Matrix, the result matrix
     */
    public Matrix mult(Matrix m) {
    	
    	// Create the new Matrix
    	MatrixStandard result = new MatrixStandard(this.name + " * " + m.getName(), this.size());
    	
    	for(int i = 1; i <= result.size(); i++) {
    		for(int j = 1; j <= result.size(); j++) {
    			
    			double sum = 0;
    			
    			for(int sumIndex = 1; sumIndex <= this.size(); sumIndex++) {
    				
    				sum = sum + this.getValue(i, sumIndex) * m.getValue(sumIndex, j);
    			}
    			
    			result.setValue(i, j, sum);
    		}
    	}
    	
    	return result;
    }
    
    /**
     * Retrieve the name of this matrix
     * @return the name
     */
    public String getName() {
        return this.name;
    }
    
    /**
     * Set a new name for this matrix
     * @param name
     */
    public void setName(String name) {
        this.name = name;
    }
    
    /**
     * Convert all data in the matrix to a quadratic double array
     * @return double[][]
     */
    public double[][] asArray() {
    	
    	// Allocate the array
    	double[][] array = new double[this.size()][this.size()];
    	
    	// Fill the array with the values
    	for(int i = 1; i <= this.size(); i++) {
    		
    		for(int j = 1; j <= this.size(); j++) {
    			
    			array[i-1][j-1] = this.getValue(i,j);
    		}
    	}
    	
    	return array;    	
    }
    
    /**
     * Retrieve the amount of required memory in this matrix (in byte)
     * @return the amount of required memory in byte
     */
    public long getRequiredMemory() {
        
        long bytes = 0;
        
//        // Add the size-integer to the required amount of bytes
//        bytes += ApplicationConstants.SIZE_INT;
        
        // Add the two-dimensional double-array to the required amount of bytes
        bytes += ApplicationConstants.SIZE_DOUBLE * (this.size * this.size);
        
//        // Add the string-name to the required amount of bytes
//        bytes += ApplicationConstants.SIZE_CHAR * this.name.length();
        
        return bytes;
    }
}
