/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * DV-Anwendungen in der Technik - Prof.Dr.Gleich - 8.Semester
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 5.0
 * Semestergroup: 8T
 */

/**
 * USABILITY:
 *  Copyright (c) 2005 
 *  Thomas Woellert (thomas@woellert.net)
 *  Stefan Weissbach (weissbst@gmx.de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package fhm.edu.fem.constants;

/**
 * Class Description:
 *		This class holds all constants associated with the whole application
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 * @author Stefan Weissbach (weissbst@gmx.de)
 */
public class ApplicationConstants {
    
	/** 
	 * To make sure noone is using this constructor.
	 * Else the compiler would create a public one.
	 * */
	private ApplicationConstants() {}
	
	/**
	 * GLOBAL
	 * For the whole Application
	 */
	
	// The version of the program
	public static String VERSION = "6.Leistungsnachweis";
	
	// Constants helping to calculate the required amount of memory in some classes (i.e. Matrix) (in byte)
	public static final int SIZE_INT = 4;
	public static final int SIZE_DOUBLE = 8;
	public static final int SIZE_CHAR = 2;
	
	// The length of the percentage information shown all over the application
	public static final int PERCENTAGE_LENGTH = 5;
	
	/**
	 * FOR FEMIN CONVERTER
	 */
	
	// The extension of the conversionFEM2toFEM3 output file
	public static final String FEM3_FILE_EXTENSION = ".fem3";
	public static final String FEM2_FILE_EXTENSION = ".fem2";
	
	/**
	 * FOR FEM STARTUP
	 * All Calculations
	 */

	// The Matrix class to use in all our calculations
	public static final int MATRIX_STANDARD = 1;
	public static final int MATRIX_HULL = 2;
	
	public static final int MATRIX_USE = MATRIX_STANDARD;
	
	/**
	 * FOR FEM STARTUP
	 * Membran Calculations
	 */
	
	// The Membran Algorithm to use in FEM4 calculations	
	public static final int MEMBRAN_JACOBI = 1;
	public static final int MEMBRAN_RAYLEIGH = 2;
	public static final int MEMBRAN_RAYLEIGH_S = 3;
	public static final int MEMBRAN_SVI = 4;
	
	public static final int MEMBRAN_USE = MEMBRAN_JACOBI;
	
	// The SVI Algorithm to use (in case you selected SVI to calculate the Membran)
	public static final int MEMBRAN_SVI_1 = 1;
	public static final int MEMBRAN_SVI_2 = 2;
	public static final int MEMBRAN_SVI_3 = 3;
	
	public static final int MEMBRAN_SVI_USE = MEMBRAN_SVI_1;
	
	// The Order (>0 means that the eigenvalues are sorted from small to large, 0 means from large to small)
	public static final int MEMBRAN_ORDER = 1;
	
	// The exactness value for the SVI and RAYLEIGH algorithms
	public static final double MEMBRAN_ALGORITHM_EXACTNESS = 5e-10;
	
	// The number of maximum iterations for SVI and RAYLEIGH algorithms
	public static final int MEMBRAN_ALGORITHM_MAX_ITERATIONS = 1000;
	
	// The number of eigenvalues calculated with the SVI and Rayleigh
	// This is also the number of eigenvalues displayed at the end of the Jacobi (which always calculates all eigenvalues)
	public static final int MEMBRAN_NUMBER_OF_EIGENVALUES = 10; 
	
	// If you want to norm the eigenvectors calculated by all MEMBRAN Algorithms enable this flag
	// you can also set a value to norm to (i.e. the value will be the max value in the vector, all others adjusted accordingly)
	public static final boolean MEMBRAN_NORM_EIGENVECTORS = true;
	public static final double MEMBRAN_NORM_EIGENVECTORS_VALUE = 100;
	
	// Set the value by which you want to shift and reshift the eigenvalues (0 means do nothing) (in case eigenvalue = 0 failes)
	public static final double MEMBRAN_SHIFT_ORIGIN = 0;
	
	// To store the machine precision
	public static final double MEMBRAN_MACHINE_PRECISION = 5e-16;
	
	/**
	 * FOR FEM5
	 */
	// Set this variable in case you want the elements to overlapp each other. The radiuses are also adjusted properly.
	public static final boolean MEMBRAN_CIRCLE_OVERLAPPING_ELEMENTS = false;
	
	/**
	 * FOR FEM6
	 */
	// The value to enlarge the physical movement of the nodes in the result so we can see it
	public static final double SPANNER_ENHANCE_MOVEMENT_FACTOR = 100;
	
	// How the element-stiffness-matrix should be calculated
	public static final int SPANNER_STIFFNESSMATRIX_SMATRICES = 1;
	public static final int SPANNER_STIFFNESSMATRIX_INTEGRAL = 2;
	
	public static final int SPANNER_STIFFNESSMATRIX_USE = SPANNER_STIFFNESSMATRIX_SMATRICES;
	
	/**
	 * FOR EIGENVALUES STARTUP
	 * Calculation of the exact eigenvalues depending on the Membran's size
	 */
	
	// To determine how many eigenvalues are needed
	public static final int EXACT_EIGENVALUES_NUMBER = 8;
	
	/**
	 * FOR ANIMATION STARTUP
	 */
	
	// The material constants
	public static final int ANIMATION_MEMBRAN_MATERIAL_CONSTANT_G = 1;
	public static final int ANIMATION_MEMBRAN_MATERIAL_CONSTANT_SIGMA = 1;
	
	// The dimensions of the displayed frame
	public static final int ANIMATION_FRAME_WIDTH = 640;
	public static final int ANIMATION_FRAME_HEIGHT = 480;
	
	/**
	 * Retrieve the help text for the application
	 * @return the help text
	 */
	public static String getHelpText() {
	    
	    // The help text string
	    String text = "";
	    
	    // Build the help text string
	    text = "Help: \nYou need to provide the following command-line arguments: \n FEMStartup <FEMIN File> <FEMOUT File>";
	    
	    return text;
	}
	
	/**
	 * Retrieve the help text for the converter
	 * @return the converter help text
	 */
	public static String getConverterHelpText() {
	    
	    // The help text string
	    String text = "";
	    
	    // Build the help text string
	    text = "Help: \nYou need to provide the following command-line arguments: \n FEMINConverterStart <FEMIN File>";
	    
	    return text;
	}
	
	/**
	 * Retrieve the help text for calculating the exact eigenvalues
	 * @return the eigenvalues help text
	 */
	public static String getEigenvaluesHelpText() {
		
	    // The help text string
	    String text = "";
	    
	    // Build the help text string
	    text = "Help: \nYou need to provide the following command-line arguments: \n EigenvaluesStartup <Positive Length Side A> <Positive Length Side B>";
	    
	    return text;
	}
	
	/**
	 * Retrieve the help text for calculating an animation
	 * @return the eigenvalues help text
	 */
	public static String getAnimationHelpText() {
		
	    // The help text string
	    String text = "";
	    
	    // Build the help text string
	    text = "Help: \nYou need to provide the following command-line arguments: \n AnimationStartup <FEMIN File> <x-value> <y-value> <z-value> <delta-T>";
	    
	    return text;
	}
	
	/**
	 * Retrieve the startup text for the application
	 * @return the startup text
	 */
	public static String getStartupText() {
	    
	    // The startup text string
	    String text = "";
	    
	    // Build the startup text string
	    text = "################################\n" + 
	    		"DV-Anwendungen in der Technik\n" +
	    		"Prof. Dr. Gleich - WS2004/05 bis SS2005\n" +
	    		"FH Muenchen\n" +
	    		"FEM Program - Version " + ApplicationConstants.VERSION + "\n" + "################################\n" +
	    				"Written by ...\n" +
	    				"Thomas Woellert (thomas@woellert.net) \n" +
	    				"Stefan Weissbach (weissbst@gmx.de)\n" + 
	    				"################################\n";
	    
	    return text;
	}
	
	/**
	 * This function converts the given number into a string and also
	 * allows the user to trim the number (most likely does not work with e-# or e+# numbers yet.
	 * It is currently only used to shorten the percentage displays
	 * @param value, the number fo convert
	 * @return String, the trimmed string representation of the number
	 */
	public static String convertPercentageToString(double value) {
		
		// Convert the Number to a normal String
		String string = Double.toString(value);
		
		// Shorten the string in case it is longer than the wanted length
		if(string.length() > ApplicationConstants.PERCENTAGE_LENGTH) {
		
			string = string.substring(0, ApplicationConstants.PERCENTAGE_LENGTH);
		}
		
		return string;
	}
}
