/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * DV-Anwendungen in der Technik - Prof.Dr.Gleich - 8.Semester
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 5.0
 * Semestergroup: 8T
 */

/**
 * USABILITY:
 *  Copyright (c) 2005 
 *  Thomas Woellert (thomas@woellert.net)
 *  Stefan Weissbach (weissbst@gmx.de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package fhm.edu.fem.constants;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import fhm.edu.fem.calc.objects.Vector;
import fhm.edu.fem.util.logging.LoggingManager;

/**
 * Class Description:
 *		The constants class holding all necessary variables for the
 *		Gauss algorithm
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 * @author Stefan Weissbach (weissbst@gmx.de)
 */
public class GaussConstants {
    
	/* Our local logger object */
	protected static Logger localLogger = LoggingManager.getLogger(GaussConstants.class.getName());
    
    /* Constants identifying the value we want */
    public static final int VALUE_A = 1;
    public static final int VALUE_H = 2;
    
    // The approach constants
    public static final int APPROACH_LINEAR = SMatrixConstants.APPROACH_LINEAR;
    public static final int APPROACH_QUADRATIC = SMatrixConstants.APPROACH_QUADRATIC;    
    
    // The figure constants
    public static final int FIGURE_TRIANGLE = SMatrixConstants.FIGURE_TRIANGLE;
    public static final int FIGURE_QUADRAT = SMatrixConstants.FIGURE_QUADRAT;
    
    /* The variables holding the constants if initialized */
	private static Vector gaussConstA2 = null;
	private static Vector gaussConstH2 = null;
	
	private static Vector gaussConstA3 = null;
	private static Vector gaussConstH3 = null;
	
	private static Vector gaussConstA4 = null;
	private static Vector gaussConstH4 = null;
	
	private static Vector gaussConstA5 = null;
	private static Vector gaussConstH5 = null;
	
	private static Vector gaussConstA6 = null;
	private static Vector gaussConstH6 = null;
	
	private static Vector gaussConstA7 = null;
	private static Vector gaussConstH7 = null;
	
	private static Vector gaussConstA8 = null;
	private static Vector gaussConstH8 = null;
	
	private static Vector gaussConstA9 = null;
	private static Vector gaussConstH9 = null;	
	
	private static Vector gaussConstA10 = null;
	private static Vector gaussConstH10 = null;
	
	/** 
	 * To make sure noone is using this constructor.
	 * Else the compiler would create a public one.
	 * */
	private GaussConstants() {}
	
	/**
	 * Calculate the D for a specific figure
	 * @param eta, the eta to use
	 * @param figure, the figure might be FIGURE_TRIANGLE or FIGURE_PARALLELOGRAM
	 * @return the result
	 */
	public static double evaluateDForFigure(double eta, int figure) throws IllegalArgumentException {
	    
	    if(figure == FIGURE_TRIANGLE) {
	        
	        return 1.0-eta;
	        
	    } else if(figure == FIGURE_QUADRAT) {
	        
	        return 1.0;
	        
	    } else {
	        
            if(localLogger.isEnabledFor(Level.ERROR)) {
                localLogger.log(Level.ERROR, "Unknown figure constant given: " + figure);
            }
            
            throw new IllegalArgumentException();
	    }
	}
	
	/**
	 * Retrieve a specific "a" or "H" constant depending on "n"
	 * @param n, which n is needed (may be 2 to 10)
	 * @param value, may be VALUE_A or VALUE_H
	 * @return the vector
	 */
	public static Vector get(int n, int value){
	    
	    // Check if we got a proper n
	    if(n > 10 || n < 2) {
	        return null;
	    }
	    
	    // Check which "n" we want the value fof	    
	    switch(n) {
		    
		    case 2:{
		        
		        if(value == VALUE_A){
		            
		            if(gaussConstA2 == null){
		                
		                gaussConstA2 =  GaussConstants.buildGaussConstA2();
		            }
		            
		            return gaussConstA2;
		        }
		        else
		        {
		            if(gaussConstH2 == null){
		                gaussConstH2 =  GaussConstants.buildGaussConstH2();
		                
	            }
		            return  gaussConstH2;
		            
		        }
		    }
		    case 3:{
		        
		        
		        if(value == VALUE_A){
		            if(gaussConstA3 == null){
		                gaussConstA2 =  GaussConstants.buildGaussConstA3();
		            }
		            return gaussConstA3;
		        }
		        else{
		            if(gaussConstH3 == null){
		                gaussConstH3 =  GaussConstants.buildGaussConstH3();
	            }
		            return  gaussConstH3;
		        }
		    }	    
		    case 4:{
		        
		        
		        if(value == VALUE_A){
		            if(gaussConstA4 == null){
		                gaussConstA4 =  GaussConstants.buildGaussConstA4();
		            }
		            return gaussConstA4;
		        }
		        else{
		            if(gaussConstH4 == null){
		                gaussConstH4 =  GaussConstants.buildGaussConstH4();
	            }
		            return  gaussConstH4;
		        }
		    }
		    case 5:{
		        
		        
		        if(value == VALUE_A){
		            if(gaussConstA5 == null){
		                gaussConstA5 =  GaussConstants.buildGaussConstA5();
		            }
		            return gaussConstA5;
		        }
		        else{
		            if(gaussConstH5 == null){
		                gaussConstH5 =  GaussConstants.buildGaussConstH5();
	            }
		            return  gaussConstH5;
		        }
		    }
		    case 6:{
		        
		        
		        if(value == VALUE_A){
		            if(gaussConstA6 == null){
		                gaussConstA6 =  GaussConstants.buildGaussConstA6();
		            }
		            return gaussConstA6;
		        }
		        else{
		            if(gaussConstH6 == null){
		                gaussConstH6 =  GaussConstants.buildGaussConstH6();
	            }
		            return  gaussConstH6;
		        }
		    }
		    case 7:{
		        
		        
		        if(value == VALUE_A){
		            if(gaussConstA7 == null){
		                gaussConstA7 =  GaussConstants.buildGaussConstA7();
		            }
		            return gaussConstA7;
		        }
		        else{
		            if(gaussConstH7 == null){
		                gaussConstH7 =  GaussConstants.buildGaussConstH7();
	            }
		            return  gaussConstH7;
		        }
		    }
		    case 8:{
		        
		        
		        if(value == VALUE_A){
		            if(gaussConstA8 == null){
		                gaussConstA8 =  GaussConstants.buildGaussConstA8();
		            }
		            return gaussConstA8;
		        }
		        else{
		            if(gaussConstH8 == null){
		                gaussConstH8 =  GaussConstants.buildGaussConstH8();
	            }
		            return  gaussConstH8;
		        }
		    }	    
		    case 9:{
		        
		        
		        if(value == VALUE_A){
		            if(gaussConstA9 == null){
		                gaussConstA9 =  GaussConstants.buildGaussConstA9();
		            }
		            return gaussConstA9;
		        }
		        else{
		            if(gaussConstH9 == null){
		                gaussConstH9 =  GaussConstants.buildGaussConstH9();
	            }
		            return  gaussConstH9;
		        }
		    }   
		    case 10:{
		        
		        
		        if(value == VALUE_A){
		            if(gaussConstA10 == null){
		                gaussConstA10 =  GaussConstants.buildGaussConstA10();
		            }
		            return gaussConstA10;
		        }
		        else{
		            if(gaussConstH10 == null){
		                gaussConstH10 =  GaussConstants.buildGaussConstH10();
	            }
		            return  gaussConstH10;
		        }
		    }
		    default:{
		        return null;
		    }
	    
	    }
	}
	
	/**
	 * Create the Vector for A2
	 * @return the new Vector
	 */
	private static Vector buildGaussConstA2(){
	    Vector tmp = new Vector("A2",2);
	    tmp.setValue(1,-0.577350269189626);
	    tmp.setValue(2,0.577350269189626);
	    return tmp;
	}
	
	/**
	 * Create the Vector for H2
	 * @return the new Vector
	 */
	private static Vector buildGaussConstH2(){
	    Vector tmp = new Vector("H2",2);
	    tmp.setValue(1,1.0);
	    tmp.setValue(2,1.0);
	    return tmp;
	}
	
	/**
	 * Create the Vector for A3
	 * @return the new Vector
	 */
	private static Vector buildGaussConstA3(){
	    Vector tmp = new Vector("A3",3);
	    tmp.setValue(1,-0.774596669241483);
	    tmp.setValue(2,0.0);
	    tmp.setValue(3,0.774596669241483);
	    return tmp;
	}
	
	/**
	 * Create the Vector for H3
	 * @return the new Vector
	 */
	private static Vector buildGaussConstH3(){
	    Vector tmp = new Vector("H3",3);
	    tmp.setValue(1,0.555555555555556);
	    tmp.setValue(2,0.888888888888889);
	    tmp.setValue(3,0.555555555555556);
	    return tmp;
	}

	/**
	 * Create the Vector for A42
	 * @return the new Vector
	 */
	private static Vector buildGaussConstA4(){
	    Vector tmp = new Vector("A4",4);
	    tmp.setValue(1,-0.861136311594053);
	    tmp.setValue(2,-0.339981043584856);
	    tmp.setValue(3,0.339981043584856);
	    tmp.setValue(4,0.86113631159405);
	    return tmp;
	}
	
	/**
	 * Create the Vector for H4
	 * @return the new Vector
	 */
	private static Vector buildGaussConstH4(){
	    Vector tmp = new Vector("H4",4);
	    tmp.setValue(1,0.347854845137454);
	    tmp.setValue(2,0.652145154862546);
	    tmp.setValue(3,0.652145154862546);
	    tmp.setValue(4,0.347854845137454);
	    return tmp;
	}
	
	/**
	 * Create the Vector for A5
	 * @return the new Vector
	 */
	private static Vector buildGaussConstA5(){
	    Vector tmp = new Vector("A5",5);
	    tmp.setValue(1,-0.906179845938664);
	    tmp.setValue(2,-0.538469310105683);
	    tmp.setValue(3,0.0);
	    tmp.setValue(4,0.538469310105683);
	    tmp.setValue(5,0.906179845938664);
	    return tmp;
	}
	
	/**
	 * Create the Vector for H5
	 * @return the new Vector
	 */
	private static Vector buildGaussConstH5(){
	    Vector tmp = new Vector("H5",5);
	    tmp.setValue(1,0.236926885056189);
	    tmp.setValue(2,0.478628670499366);
	    tmp.setValue(3,0.568888888888889);
	    tmp.setValue(4,0.478628670499366);
	    tmp.setValue(5,0.236926885056189);
	    return tmp;
	}
	
	/**
	 * Create the Vector for A6
	 * @return the new Vector
	 */
	private static Vector buildGaussConstA6(){
	    Vector tmp = new Vector("A6",6);
	    tmp.setValue(1,-0.932469514203152);
	    tmp.setValue(2,-0.661209386466265);
	    tmp.setValue(3,-0.238619186083197);
	    tmp.setValue(4,0.238619186083197);
	    tmp.setValue(5,0.661209386466265);
	    tmp.setValue(6,0.932469514203152);
	    return tmp;
	}
	
	/**
	 * Create the Vector for H6
	 * @return the new Vector
	 */
	private static Vector buildGaussConstH6(){
	    Vector tmp = new Vector("H6",6);
	    tmp.setValue(1,0.171324492379170);
	    tmp.setValue(2,0.360761573048139);
	    tmp.setValue(3,0.467913934572691);
	    tmp.setValue(4,0.467913934572691);
	    tmp.setValue(5,0.360761573048139);
	    tmp.setValue(6,0.171324492379170);
	    return tmp;
	}
	
	/**
	 * Create the Vector for A7
	 * @return the new Vector
	 */
	private static Vector buildGaussConstA7(){
	    Vector tmp = new Vector("A7",7);
	    tmp.setValue(1,-0.949107912342759);
	    tmp.setValue(2,-0.741531185599394);
	    tmp.setValue(3,-0.405845151377397);
	    tmp.setValue(4,0.0);
	    tmp.setValue(5,0.405845151377397);
	    tmp.setValue(6,0.741531185599394);
	    tmp.setValue(7,0.949107912342759);
	    return tmp;
	}
	
	/**
	 * Create the Vector for H7
	 * @return the new Vector
	 */
	private static Vector buildGaussConstH7(){
	    Vector tmp = new Vector("H7",7);
	    tmp.setValue(1,0.129484966168870);
	    tmp.setValue(2,0.279705391489277);
	    tmp.setValue(3,0.381830050505119);
	    tmp.setValue(4,0.417959183673469);
	    tmp.setValue(5,0.381830050505119);
	    tmp.setValue(6,0.279705391489277);
	    tmp.setValue(7,0.129484966168870);
	    return tmp;
	}
	
	/**
	 * Create the Vector for A8
	 * @return the new Vector
	 */
	private static Vector buildGaussConstA8(){
	    Vector tmp = new Vector("A8",8);
	    tmp.setValue(1,-0.960289856497536);
	    tmp.setValue(2,-0.796666477413627);
	    tmp.setValue(3,-0.525532409916329);
	    tmp.setValue(4,-0.183434642495650);
	    tmp.setValue(5,0.183434642495650);
	    tmp.setValue(6,0.525532409916329);
	    tmp.setValue(7,0.796666477413627);
	    tmp.setValue(8,0.960289856497536);
	    return tmp;
	}
	
	/**
	 * Create the Vector for H8
	 * @return the new Vector
	 */
	private static Vector buildGaussConstH8(){
	    Vector tmp = new Vector("H8",8);
	    tmp.setValue(1,0.101228536290376);
	    tmp.setValue(2,0.222381034453374);
	    tmp.setValue(3,0.313706645877887);
	    tmp.setValue(4,0.362683783378362);
	    tmp.setValue(5,0.362683783378362);
	    tmp.setValue(6,0.313706645877887);
	    tmp.setValue(7,0.222381034453374);
	    tmp.setValue(8,0.101228536290376);
	    return tmp;
	}
	
	/**
	 * Create the Vector for A9
	 * @return the new Vector
	 */
	private static Vector buildGaussConstA9(){
	    Vector tmp = new Vector("A9",9);
	    tmp.setValue(1,-0.968160239507626);
	    tmp.setValue(2,-0.836031107326636);
	    tmp.setValue(3,-0.613371432700590);
	    tmp.setValue(4,-0.432253423403809);
	    tmp.setValue(5,0.0);
	    tmp.setValue(6,0.432253423403809);
	    tmp.setValue(7,0.613371432700590);
	    tmp.setValue(8,0.836031107326636);
	    tmp.setValue(9,0.968160239507626);
	    return tmp;
	}
	
	/**
	 * Create the Vector for H9
	 * @return the new Vector
	 */
	private static Vector buildGaussConstH9(){
	    Vector tmp = new Vector("H9",9);
	    tmp.setValue(1,0.081274388361574);
	    tmp.setValue(2,0.180648160694857);
	    tmp.setValue(3,0.260610696402935);
	    tmp.setValue(4,0.312347077040003);
	    tmp.setValue(5,0.330239355001260);
	    tmp.setValue(6,0.312347077040003);
	    tmp.setValue(7,0.260610696402935);
	    tmp.setValue(8,0.180648160694857);
	    tmp.setValue(9,0.081274388361574);
	    return tmp;
	}
	
	/**
	 * Create the Vector for A10
	 * @return the new Vector
	 */
	private static Vector buildGaussConstA10(){
	    Vector tmp = new Vector("A10",10);
	    tmp.setValue(1,-0.973906528517172);
	    tmp.setValue(2,-0.865063366688985);
	    tmp.setValue(3,-0.679409568299024);
	    tmp.setValue(4,-0.433395394129247);
	    tmp.setValue(5,-0.148874338981631);
	    tmp.setValue(6,0.148874338981631);
	    tmp.setValue(7,0.433395394129247);
	    tmp.setValue(8,0.679409568299024);
	    tmp.setValue(9,0.865063366688985);
	    tmp.setValue(10,0.973906528517172);
	    return tmp;
	}
	
	/**
	 * Create the Vector for H10
	 * @return the new Vector
	 */
	private static Vector buildGaussConstH10(){
	    Vector tmp = new Vector("H10",10);
	    tmp.setValue(1, 0.066671344308688);
	    tmp.setValue(2, 0.149451349150581);
	    tmp.setValue(3, 0.219086362515982);
	    tmp.setValue(4, 0.269266719309996);
	    tmp.setValue(5 ,0.295524224714753);
	    tmp.setValue(6, 0.295524224714753);
	    tmp.setValue(7, 0.269266719309996);
	    tmp.setValue(8, 0.219086362515982);
	    tmp.setValue(9, 0.149451349150581);
	    tmp.setValue(10, 0.066671344308688);
	    return tmp;
	}
}
