/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * DV-Anwendungen in der Technik - Prof.Dr.Gleich - 8.Semester
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 5.0
 * Semestergroup: 8T
 */

/**
 * USABILITY:
 *  Copyright (c) 2005 
 *  Thomas Woellert (thomas@woellert.net)
 *  Stefan Weissbach (weissbst@gmx.de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package fhm.edu.fem.test;

/**
 * Class Description:
 *		Test Class - Envelope Data Item
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 * @author Stefan Weissbach (weissbst@gmx.de)
 */
public class ENVDataItem {
    
	/* The Key by which we can identify this object despite it's value */
	Comparable key;

    protected ENVDataItem prev = null;
    protected ENVDataItem next = null;
    protected double value = 0.0;
    
    public ENVDataItem(double value) {
        this.value = value;
        this.key = this.generateKey();
    }
    
    public void setValue(double value) {
        this.value = value;
    }
    
    public double getValue() {
        return this.value;
    }
    
    public void setPrevious(ENVDataItem prev) {
        this.prev = prev;
    }
    
    public ENVDataItem getPrevious() {
        return this.prev;
    }
    
    public void setNext(ENVDataItem next) {
        this.next = next;
    }
    
    public ENVDataItem getNext() {
        return this.next;
    }
    
	protected Comparable generateKey() {
		int result = this.hashCode() + new Long( System.currentTimeMillis() ).intValue();
		return new Integer(result);
	}
	
	public Comparable getKey() {
	    return this.key;
	}
    
    /* (non-Javadoc)
     * @see java.lang.Object#equals(java.lang.Object)
     */
    public boolean equals(Object obj) {
        
        // Check if the given object is of the same class as this one
        if(obj instanceof ENVDataItem) {
            
            // Compare the two keys of the objects
            if(this.key.compareTo(((ENVDataItem)obj).getKey()) == 0) {
                
                // The keys match
                return true;
                
            } else {
                
                // The keys don't match
                return false;
            }
            
        } else {
            
            // The given object is not even of the same type as this one, so it is not equal
            return false;
        }
    }
    
    public String toString() {
        return "[ENVDataItem: value= " + this.value + "]";
    }
    
    
    /* (non-Javadoc)
     * @see java.lang.Object#clone()
     */
    protected Object clone() throws CloneNotSupportedException {
        
        return new ENVDataItem(this.value);
    }
}
