/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * DV-Anwendungen in der Technik - Prof.Dr.Gleich - 8.Semester
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 5.0
 * Semestergroup: 8T
 */

/**
 * USABILITY:
 *  Copyright (c) 2005 
 *  Thomas Woellert (thomas@woellert.net)
 *  Stefan Weissbach (weissbst@gmx.de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package fhm.edu.fem.test;

import java.lang.reflect.Method;

import fhm.edu.fem.calc.FEMGaussQuadrature;
import fhm.edu.fem.calc.objects.MathObject;
import fhm.edu.fem.calc.objects.Vector;
import fhm.edu.fem.constants.GaussConstants;

/**
 * Class Description:
 *		Test Class - Single Integration
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 * @author Stefan Weissbach (weissbst@gmx.de)
 */
public class TestSingleIntegration {

    public static void main(String[] args) {
    	
        try {
			double a = 0;
			double b = 1;
			Method h1 = TestSingleIntegration.getMethod("xSquared");
			Method h2 = TestSingleIntegration.getMethod("one");
			Method q = TestSingleIntegration.getMethod("one");
			int z1 = 1;
			Vector ay = GaussConstants.get(6, GaussConstants.VALUE_A);
			Vector hy = GaussConstants.get(6, GaussConstants.VALUE_H);
			MathObject integralResultObject = null;
			
			// Calculate the single gauss quadrature
			integralResultObject = FEMGaussQuadrature.calcSingleIntegration(a, b, h1, h2, q, z1, z1, ay, hy);
			
			System.err.println(integralResultObject);
			
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
    }
    
	/**
	 * Get the proper method
	 * @param methodName, the method name taken from the constants in this class
	 * @return the method
	 */
	public static Method getMethod(String methodName) throws SecurityException, NoSuchMethodException {
	    
	    // Build the args array
	    Class args[] = new Class[3];
	    args[0] = Double.class;
	    args[1] = Double.class;
	    args[2] = Integer.class;
	    
	    return TestSingleIntegration.class.getDeclaredMethod(methodName, args);
	}
	
	/**
	 * Helper Method
	 * @param xi
	 * @param eta
	 * @param position
	 * @return
	 */
	public static double xSquared(Double xi, Double eta, Integer position) {
	    
	    return (xi.doubleValue()*xi.doubleValue());
	}
	
	/**
	 * Helper Method
	 * @param xi
	 * @param eta
	 * @param position
	 * @return
	 */
	public static double one(Double xi, Double eta, Integer position) {
	    
	    return 1.0;
	}
}
