/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * DV-Anwendungen in der Technik - Prof.Dr.Gleich - 8.Semester
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 5.0
 * Semestergroup: 8T
 */

/**
 * USABILITY:
 *  Copyright (c) 2005 
 *  Thomas Woellert (thomas@woellert.net)
 *  Stefan Weissbach (weissbst@gmx.de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package fhm.edu.fem.util.io;

import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.StringWriter;
import java.util.List;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import fhm.edu.fem.calc.FEMManager;
import fhm.edu.fem.calc.objects.Vector;
import fhm.edu.fem.util.logging.LoggingManager;

/**
 * Class Description:
 *		This class takes care of saving the results to the FEMOUT file
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 * @author Stefan Weissbach (weissbst@gmx.de)
 */
public class FEMOUTSaver {
    
	/* Our local logger object */
	protected static Logger localLogger = LoggingManager.getLogger(FEMOUTSaver.class.getName());

    /**
     * Construct a new FEMOUTSaver
     */
    public FEMOUTSaver() {}
    
    /**
     * Save all results (which are stored in the FEMManager) to the given file
     * @param outFile, the file to save to
     */
    public void save(File outFile) {
        
        try {
            
            // Open the file output stream
            FileOutputStream outputStream = new FileOutputStream(outFile, false);
            
            // Grab the results from the FEMManager
            List<Vector> resultList = FEMManager.getInstance().getResultList();
            
            // Create a new StringWriter
            StringWriter writer = new StringWriter();
            
            // Write # to make it compatible with the fem_plotter
            writer.write("#\n");
            
            // Go through all results
            for(int i = 0; i < resultList.size(); i++) {
                
                // Write the header line
                writer.write("# Results after Condition: " + (i+1) + "\n");
                
                // Grab the result
                Vector u =  resultList.get(i);
                
                // Run through vector u
                for(int j = 1; j <= u.size(); j++) {
                    
                    double value = u.getValue(j);
                    
                    // Save the value in the writer
                    writer.write(Double.toString(value));
                    
                    // Only write a whitespace between the numbers, not at the end
                    if(j != u.size()) {
                        writer.write(" ");
                    }
                }
                
                // Add a "new line" if there is another result Vector u following
                if((i+1) < resultList.size()) {
                    writer.write("\n");
                }
            }
            
            // Grab the buffer from the writer
            StringBuffer buffer = writer.getBuffer();
            String string = buffer.toString();
            
            // Create an OutputStreamWriter
            OutputStreamWriter outputStreamWriter = new OutputStreamWriter(outputStream);
            
            // Write the String to the output stream and close it
            outputStreamWriter.write(string);
            outputStreamWriter.close();            
            
        } catch (Exception e) {
            
			if(localLogger.isEnabledFor(Level.ERROR)) {
				localLogger.log(Level.ERROR, "Loading file failed: " + outFile.getName(), e);
			}
        }
    }
}
