/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * DV-Anwendungen in der Technik - Prof.Dr.Gleich - 8.Semester
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 5.0
 * Semestergroup: 8T
 */

/**
 * USABILITY:
 *  Copyright (c) 2005 
 *  Thomas Woellert (thomas@woellert.net)
 *  Stefan Weissbach (weissbst@gmx.de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package fhm.edu.fem.util.logging;

import java.util.Properties;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.apache.log4j.PropertyConfigurator;

/**
 * Class Description:
 * 		Based on the log4j package. Takes care of initializing itself
 * 		via the properties. This singleton should always be asked for a logger.
 * 		Ensures that all requested loggers use the same configuration
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 * @author Stefan Weissbach (weissbst@gmx.de)
 */
public class LoggingManager {
	
	/* The instance of our LoggingManager, to make sure that there is only one */
	protected static LoggingManager theInstance = new LoggingManager();
	
	/* The LogManager we are using */
	protected static LogManager logManager = null;
	
	/* The properties we configured the logging manager with */
	protected static Properties props = null;
	
	/* Are we logging in this class? */
	protected static boolean isLogging = false;
	
	/**
	 * Private constructor, enforces singleton
	 * DO NOT FORGET TO SET THE PROPERTIES
	 * */
	private LoggingManager() {}
	
	/**
	 * Give the LoggingManager the properties object
	 * @param props, the properties object holding the configuration information for log4j
	 */
	public static void setProperties(Properties props) {
		LoggingManager.props = props;
		LoggingManager.init();
	}
	
	/**
	 * Initialize the logging manager properly
	 */
	protected static void init() {
		try {
			// Initialize the log configuration from the properties file
			PropertyConfigurator.configure(LoggingManager.props);
			
			// Initialize our LogManager object
			logManager = new LogManager();
		} catch (Exception e) {
			if(LoggingManager.isLogging) {
				System.out.println("[FATAL] - LoggingManager - Creating logging manager failed");
			}
		}
	}
	
	/**
	 * Create a new logger with the given name from our LoggingManager
	 * @param loggerName, the name of the logger we requested. Typically the package/classname, i.e. examples.MyExample
	 * */
    public static Logger getLogger(String loggerName) {
    	if(logManager != null) {
        	Logger newLogger = null;
        	
        	try {
        		newLogger = LogManager.getLogger(loggerName);
        	} catch (Exception e) {
        		if(LoggingManager.isLogging) {
        			System.out.println("[FATAL] - LoggingManager - Retrieving/Creating new logger " + loggerName + " failed");
        		}
        	}
        	
        	return newLogger;
    	} else {
    		if(LoggingManager.isLogging) {
    			System.out.println("[FATAL] - LoggingManager - Cannot create a logger named: " + loggerName + ". No properties for configuration set !");
    		}
    		return null;
    	}
    }
    
    /** 
     * Get the sole instance 
     * @return theInstance
     * */
    public static LoggingManager getInstance() {
       return theInstance;
    }
}
