/**
 * Munich University of Applied Sciences, FB07 Computer Science
 * DV-Anwendungen in der Technik - Prof.Dr.Gleich - 8.Semester
 * 
 * System: Windows XP, Linux 2.6, Sun Java JDK 5.0
 * Semestergroup: 8T
 */

/**
 * USABILITY:
 *  Copyright (c) 2005 
 *  Thomas Woellert (thomas@woellert.net)
 *  Stefan Weissbach (weissbst@gmx.de)
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation the
 * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 * sell copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 * IN THE SOFTWARE.
 */

package fhm.edu.fem.util.properties;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.LinkedHashMap;
import java.util.Properties;

/**
 * Class Description:
 * 		This class read it's input from a given properties file and caches the data in a hash map.
 * 
 * @author Thomas Woellert (thomas@woellert.net)
 * @author Stefan Weissbach (weissbst@gmx.de)
 */
public class PropertyCache {
	
	/* Internal cache for properties. */
	protected LinkedHashMap<String,Properties> propertyMap = new LinkedHashMap<String,Properties>();
	
	/* The only one static instance of PropertiesCache */
	protected static PropertyCache theInstance = new PropertyCache();
	
	/* Are we logging in this class? */
	protected static boolean isLogging = false;
	
	/**
	 * Construct a new property cache
	 * Protected to enforce singleton
	 */
	protected PropertyCache() {
		if(PropertyCache.isLogging) {
			System.out.println("Creating PropertyCache");
		}
	}
	
	/**
	 * Retrieve the sole instance of this property cache
	 * @return theInstance
	 */
	public static PropertyCache getInstance() {
		return theInstance;
	}

	/** 
	 * Retrieve the properties for the given filename
	 * @param fileName, the filename pointing to the properties file
	 */
	public static Properties getProperties(String fileName) {
		
		// Check if resource ist already loaded
		if ( getInstance().propertyMap.containsKey(fileName)) {
			if(PropertyCache.isLogging) {
				System.out.println("Get file [" + fileName + "] from cache");
			}
			return (Properties) getInstance().propertyMap.get(fileName);
		} else {
			if(PropertyCache.isLogging) {
				System.out.println("Open new file [" + fileName + "]" );
			}

			// Load
			Properties propsIn = new Properties();

			try {
				FileInputStream inputStream = new FileInputStream(fileName);
				propsIn.load(inputStream);
			} catch (FileNotFoundException e) {
				if(PropertyCache.isLogging) {
					System.err.println("Caught FileNotFoundException loading file: "	+ fileName);
				}
			} catch (IOException e) {
				if(PropertyCache.isLogging) {
					System.err.println("Caught IOException loading file: " + fileName);
					System.err.println("Using default properties");
				}
			}

			// Put into cache
			getInstance().propertyMap.put(fileName, propsIn);
			return propsIn;
		}
	}
	
	
	/**
	 * Get the string value from given key over all the cached key-value-pairs
	 * inside given file resource
	 * @param filename The properties file containing the property
	 * @param key The key of the property
	 * @return String The stringified property value
	 */
	public static final String getProperty( final String fileName, final String key ) {
		return getProperties(fileName).getProperty(key );		
	}
}
